! =================================================================================================================================
! MODULE       : sapiens_lcchange
!
! CONTACT      : orchidee-help _at_ ipsl.jussieu.fr
!
! LICENCE      : IPSL (2006)
! This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF       Impact of land cover change on carbon stocks
!!
!!\n DESCRIPTION: None
!!
!! RECENT CHANGE(S): None
!!
!! REFERENCE(S)	: None
!!
!! SVN          :
!! $HeadURL: svn://forge.ipsl.jussieu.fr/orchidee/branches/ORCHIDEE-DOFOCO/ORCHIDEE/src_stomate/stomate_lcchange.f90 $
!! $Date: 2013-07-03 10:16:15 +0200 (Wed, 03 Jul 2013) $
!! $Revision: 1356 $
!! \n
!_ ================================================================================================================================


MODULE sapiens_lcchange

  ! modules used:
  
  USE ioipsl_para
  USE stomate_data
  USE pft_parameters
  USE constantes
  USE stomate_prescribe
  USE function_library, only: wood_to_dia, wood_to_ba, check_biomass_sync 
  
  IMPLICIT NONE
  
  PRIVATE
  PUBLIC lcchange_main, land_cover_change_main, age_class_distr, merge_biomass_pfts
  
CONTAINS


!! ================================================================================================================================
!! SUBROUTINE   : lcchange_main
!!
!>\BRIEF        Impact of land cover change on carbon stocks
!!
!! DESCRIPTION  : This subroutine is activated by 'LAND_COVER_CHANGE = y' in configuration file.
!! In the case of 'LAND_COVER_CHANGE = y', the impact of land cover change on 
!! carbon stocks is computed in this subroutine. The land cover change is written
!! by the difference of new and previous "maximal" coverage fraction of a PFT. 
!! On the basis of this difference, the amount of 'new establishment'/'biomass export',
!! and increase/decrease of each component, are estimated.\n
!!
!! Main structure of stomate_lcchange.f90 is:
!! 1. Initialization
!! 2. Calculation of changes in carbon stocks and biomass by land cover change
!! 3. Update 10 year- and 100 year-turnover pool contents
!! 4. History
!!
!! RECENT CHANGE(S) : None
!!
!! MAIN OUTPUT VARIABLE(S) : ::prod10, ::prod100, ::flux10, ::flux100,
!!   :: cflux_prod10 and :: cflux_prod100 
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \latexonly 
!!     \includegraphics[scale=0.5]{lcchange.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE lcchange_main ( npts, dt_days, veget_max, veget_max_new,&
       biomass, ind, age, PFTpresent, &
       senescence, when_growthinit, everywhere, co2_to_bm, &
       bm_to_litter, turnover_daily, cn_ind, flux10, &
       flux100, prod10, prod100, convflux, &
       cflux_prod10, cflux_prod100, leaf_frac, npp_longterm, &
       lm_lastyearmax, litter, carbon, lignin_struc, &
       lignin_wood )
    
    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
    
    INTEGER, INTENT(in)                                       :: npts             !! Domain size - number of pixels (unitless)
    REAL(r_std), INTENT(in)                                   :: dt_days          !! Time step of vegetation dynamics for stomate
                                                                                  !! (days)
 
    !! 0.2 Output variables

    REAL(r_std), DIMENSION(npts), INTENT(out)                 :: convflux         !! release during first year following land 
                                                                                  !! cover change
    REAL(r_std), DIMENSION(npts), INTENT(out)                 :: cflux_prod10     !! total annual release from the 10 year 
                                                                                  !! turnover pool @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts), INTENT(out)                 :: cflux_prod100    !! total annual release from the 100 year-
                                                                                  !! turnover pool @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                               INTENT(inout)  :: turnover_daily   !! Turnover rates 
                                                                                  !! @tex ($gC m^{-2} day^{-1}$) @endtex

    !! 0.3 Modified variables   
    
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: veget_max        !! "maximal" coverage fraction of a PFT (LAI ->
                                                                                  !! infinity) on ground.  (unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: veget_max_new    !! new "maximal" coverage fraction of a PFT (LAI
                                                                                  !! -> infinity) on ground. 
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                               INTENT(inout)  :: biomass          !! biomass @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: ind              !! Number of individuals @tex ($m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: age              !! mean age (years)
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)               :: senescence       !! plant senescent (only for deciduous trees) Set
                                                                                  !! to .FALSE. if PFT is introduced or killed
    LOGICAL, DIMENSION(npts,nvm), INTENT(inout)               :: PFTpresent       !! Is pft there (unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: everywhere       !! is the PFT everywhere in the grid box or very 
                                                                                  !! localized (unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: when_growthinit  !! how many days ago was the beginning of the 
                                                                                  !! growing season (days)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: co2_to_bm        !! biomass uptaken 
                                                                                  !! @tex ($gC m^{-2} day^{-1}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements), &
                                              INTENT(inout)   :: bm_to_litter     !! conversion of biomass to litter 
                                                                                  !! @tex ($gC m^{-2} day^{-1}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: cn_ind           !! crown area of individuals 
                                                                                  !! @tex ($m^{2}$) @endtex
    REAL(r_std), DIMENSION(npts,0:10), INTENT(inout)          :: prod10           !! products remaining in the 10 year-turnover
                                                                                  !! pool after the annual release for each 
                                                                                  !! compartment (10 + 1 : input from year of land
                                                                                  !! cover change)
    REAL(r_std), DIMENSION(npts,0:100), INTENT(inout)         :: prod100          !! products remaining in the 100 year-turnover
                                                                                  !! pool after the annual release for each 
                                                                                  !! compartment (100 + 1 : input from year of land
                                                                                  !! cover change)
    REAL(r_std), DIMENSION(npts,10), INTENT(inout)            :: flux10           !! annual release from the 10/100 year-turnover 
                                                                                  !! pool compartments
    REAL(r_std), DIMENSION(npts,100), INTENT(inout)           :: flux100          !! annual release from the 10/100 year-turnover
                                                                                  !! pool compartments
    REAL(r_std), DIMENSION(npts,nvm,nleafages), INTENT(inout) :: leaf_frac        !! fraction of leaves in leaf age class 
                                                                                  !! (unitless)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: lm_lastyearmax   !! last year's maximum leaf mass for each PFT 
                                                                                  !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)           :: npp_longterm     !! "long term" net primary productivity 
                                                                                  !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std),DIMENSION(npts,nlitt,nvm,nlevs,nelements), &
                                              INTENT(inout)   :: litter           !! metabolic and structural litter, above and 
                                                                                  !! below ground @tex ($gC m^{-2}$) @endtex
    REAL(r_std),DIMENSION(npts,ncarb,nvm), INTENT(inout)      :: carbon           !! carbon pool: active, slow, or passive 
                                                                                  !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)     :: lignin_struc     !! ratio Lignine/Carbon in structural litter,
                                                                                  !! above and below ground
    REAL(r_std), DIMENSION(npts,nvm,nlevs), INTENT(inout)     :: lignin_wood      !! ratio Lignine/Carbon in woody litter,
                                                                                  !! above and below ground


    !! 0.4 Local variables

    INTEGER(i_std)                                            :: i, j, k, l, m    !! indices (unitless)
    REAL(r_std),DIMENSION(npts,nelements)                     :: bm_new           !! biomass increase @tex ($gC m^{-2}$) @endtex
    REAL(r_std),DIMENSION(npts,nparts,nelements)              :: biomass_loss     !! biomass loss @tex ($gC m^{-2}$) @endtex
    REAL(r_std)                                               :: above            !! aboveground biomass @tex ($gC m^{-2}$) @endtex
    REAL(r_std),DIMENSION(npts,nlitt,nlevs,nelements)         :: dilu_lit         !! Litter dilution @tex ($gC m^{-2}$) @endtex
    REAL(r_std),DIMENSION(npts,ncarb)                         :: dilu_soil_carbon !! Soil Carbondilution @tex ($gC m^{-2}$) @endtex
    REAL(r_std),DIMENSION(npts,nlevs)                         :: dilu_lf_struc    !! fraction of structural litter that is lignin
                                                                                  !! (0-1,unitless)
    REAL(r_std),DIMENSION(npts,nlevs)                         :: dilu_lf_wood     !! fraction of woody litter that is lignin
                                                                                  !! (0-1,unitless)
    REAL(r_std),DIMENSION(nvm)                                :: delta_veg        !! changes in "maximal" coverage fraction of PFT 
    REAL(r_std)                                               :: delta_veg_sum    !! sum of delta_veg
    REAL(r_std),DIMENSION(npts,nvm)                           :: delta_ind        !! change in number of individuals  
!_ ================================================================================================================================

    IF (bavard.GE.3) WRITE(numout,*) 'Entering lcchange_main'
    
  !! 1. initialization
    
    prod10(:,0)         = zero
    prod100(:,0)        = zero   
    above               = zero
    convflux(:)         = zero
    cflux_prod10(:)     = zero
    cflux_prod100(:)    = zero
    delta_ind(:,:)      = zero
    delta_veg(:)        = zero
    
  !! 2. calculation of changes in carbon stocks and biomass by land cover change\n
    
    DO i = 1, npts ! Loop over # pixels - domain size
       
       !! 2.1 initialization of carbon stocks\n
       delta_veg(:) = veget_max_new(i,:)-veget_max(i,:)
       delta_veg_sum = SUM(delta_veg,MASK=delta_veg.LT.0.)
       
       dilu_lit(i,:,:,:) = zero
       dilu_soil_carbon(i,:) = zero
       biomass_loss(i,:,:) = zero
       dilu_lf_struc(i,:) = zero
       dilu_lf_wood(i,:) = zero
       
       !! 2.2 if vegetation coverage decreases, compute dilution of litter, soil carbon, and biomass.\n
       DO j=2, nvm
          IF ( delta_veg(j) < -min_stomate ) THEN 
             dilu_lit(i,:,:,:) = dilu_lit(i,:,:,:) + delta_veg(j)*litter(i,:,j,:,:) / delta_veg_sum
             dilu_soil_carbon(i,:) =  dilu_soil_carbon(i,:) + delta_veg(j) * carbon(i,:,j) / delta_veg_sum
             dilu_lf_struc(i,:) = dilu_lf_struc(i,:) + &
                  delta_veg(j) * lignin_struc(i,j,:)* litter(i,istructural,j,:,icarbon) / delta_veg_sum
             dilu_lf_wood(i,:) = dilu_lf_wood(i,:) + &
                  delta_veg(j) * lignin_wood(i,j,:)*litter(i,iwoody,j,:,icarbon) / delta_veg_sum
             biomass_loss(i,:,:) = biomass_loss(i,:,:) + biomass(i,j,:,:)*delta_veg(j) / delta_veg_sum
          ENDIF
       ENDDO
       
       !! 2.3 
       DO j=2, nvm ! Loop over # PFTs

          !! 2.3.1 The case that vegetation coverage of PFTj increases
          IF ( delta_veg(j) > min_stomate) THEN

             !! 2.3.1.1 Initial setting of new establishment
             IF (veget_max(i,j) .LT. min_stomate) THEN 
                IF (is_tree(j)) THEN

                   ! cn_sapl(j)=0.5; stomate_data.f90
                   cn_ind(i,j) = cn_sapl(j) 
                ELSE
                   cn_ind(i,j) = un
                ENDIF
                ind(i,j)= delta_veg(j) / cn_ind(i,j)
                PFTpresent(i,j) = .TRUE.
                everywhere(i,j) = 1.
                senescence(i,j) = .FALSE.
                age(i,j) = zero

                ! large_value = 1.E33_r_std
                when_growthinit(i,j) = large_value 
                leaf_frac(i,j,1) = 1.0
                npp_longterm(i,j) = npp_longterm_init
                !+++CHECK+++
                ! restore link with bm_sapl - prescribed something as a quick fix
                !lm_lastyearmax(i,j) = bm_sapl(j,ileaf,icarbon) * ind(i,j)
                lm_lastyearmax(i,j) = 5 * ind(i,j)
                !+++++++++++
                
             ENDIF
             IF ( cn_ind(i,j) > min_stomate ) THEN
                delta_ind(i,j) = delta_veg(j) / cn_ind(i,j) 
             ENDIF
             
             !! 2.3.1.2 Update of biomass in each each carbon stock component 
             !!         Update of biomass in each each carbon stock component (leaf, sapabove, sapbelow,
             !>         heartabove, heartbelow, root, fruit, and carbres)\n
             DO k = 1, nparts ! loop over # carbon stock components, nparts = 8; stomate_constant.f90 
                DO l = 1,nelements ! loop over # elements
               
                   !+++CHECK+++
                   ! restore link with bm_sapl - prescribed something as a quick fix
                   ! bm_new(i,l) = delta_ind(i,j) * bm_sapl(j,k,l)
                   bm_new(i,l) = delta_ind(i,j) * 5
                   !+++++++++++

                   IF (veget_max(i,j) .GT. min_stomate) THEN

                      ! in the case that bm_new is overestimated compared with biomass?
                      IF ((bm_new(i,l)/delta_veg(j)) > biomass(i,j,k,l)) THEN
                         bm_new(i,l) = biomass(i,j,k,l)*delta_veg(j)
                      ENDIF
                   ENDIF
                   biomass(i,j,k,l) = ( biomass(i,j,k,l) * veget_max(i,j) + bm_new(i,l) ) / veget_max_new(i,j)
                   co2_to_bm(i,j) = co2_to_bm(i,j) + (bm_new(i,icarbon)* dt_days) / (one_year * veget_max_new(i,j))
                END DO ! loop over # elements
             ENDDO ! loop over # carbon stock components

             !! 2.3.1.3 Calculation of dilution in litter, soil carbon, and  input of litter
             !!        In this 'IF statement', dilu_* is zero. Formulas for litter and soil carbon
             !!         could be shortend?? Are the following formulas correct?

            
             ! Lignin fraction of structural litter
             lignin_struc(i,j,:)=(lignin_struc(i,j,:) * veget_max(i,j)* litter(i,istructural,j,:,icarbon) + & 
                  dilu_lf_struc(i,:) * delta_veg(j)) / veget_max_new(i,j) 

             ! Lignin fraction of woody litter
             lignin_wood(i,j,:)=(lignin_wood(i,j,:) * veget_max(i,j)* litter(i,iwoody,j,:,icarbon) + & 
                  dilu_lf_wood(i,:) * delta_veg(j)) / veget_max_new(i,j)

             ! Litter
             litter(i,:,j,:,:)=(litter(i,:,j,:,:) * veget_max(i,j) + &
                  dilu_lit(i,:,:,:) * delta_veg(j)) / veget_max_new(i,j)

             WHERE ( litter(i,istructural,j,:,icarbon) > min_stomate )
                lignin_struc(i,j,:) = lignin_struc(i,j,:)/litter(i,istructural,j,:,icarbon)
             ELSEWHERE
                lignin_struc(i,j,:) = LC(ileaf)
             ENDWHERE

             WHERE ( litter(i,iwoody,j,:,icarbon) > min_stomate )
                lignin_struc(i,j,:) = lignin_struc(i,j,:)/litter(i,iwoody,j,:,icarbon)
             ELSEWHERE
                lignin_struc(i,j,:) = LC(iheartabove)
             ENDWHERE

             ! Soil carbon
             carbon(i,:,j)=(carbon(i,:,j) * veget_max(i,j) + dilu_soil_carbon(i,:) * delta_veg(j)) / veget_max_new(i,j)

             DO l = 1,nelements

                ! Litter input
                bm_to_litter(i,j,isapbelow,l) = bm_to_litter(i,j,isapbelow,l) + &
                     biomass_loss(i,isapbelow,l)*delta_veg(j) / veget_max_new(i,j)

                bm_to_litter(i,j,iheartbelow,l) = bm_to_litter(i,j,iheartbelow,l) + & 
                     biomass_loss(i,iheartbelow,l) *delta_veg(j) / veget_max_new(i,j)

                bm_to_litter(i,j,iroot,l) = bm_to_litter(i,j,iroot,l) + &
                     biomass_loss(i,iroot,l)*delta_veg(j) / veget_max_new(i,j)

                bm_to_litter(i,j,ifruit,l) = bm_to_litter(i,j,ifruit,l) + & 
                     biomass_loss(i,ifruit,l)*delta_veg(j) / veget_max_new(i,j)

                bm_to_litter(i,j,icarbres,l) = bm_to_litter(i,j,icarbres,l) + &
                     biomass_loss(i,icarbres,l)   *delta_veg(j) / veget_max_new(i,j)

                bm_to_litter(i,j,ileaf,l) = bm_to_litter(i,j,ileaf,l) + &
                     biomass_loss(i,ileaf,l)*delta_veg(j) / veget_max_new(i,j)

             END DO

             age(i,j)=age(i,j)*veget_max(i,j)/veget_max_new(i,j)
             
          !! 2.3.2 The case that vegetation coverage of PFTj is no change or decreases
          ELSE 
 
             !! 2.3.2.1 Biomass export
             ! coeff_lcchange_*:  Coeff of biomass export for the year, decade, and century
             above = biomass(i,j,isapabove,icarbon) + biomass(i,j,iheartabove,icarbon)
             convflux(i)  = convflux(i)  - ( coeff_lcchange_s(j) * above * delta_veg(j) ) 
             prod10(i,0)  = prod10(i,0)  - ( coeff_lcchange_m(j) * above * delta_veg(j) )
             prod100(i,0) = prod100(i,0) - ( coeff_lcchange_l(j) * above * delta_veg(j) )
             
             !! 2.3.2.2 Total reduction
             IF ( veget_max_new(i,j) .LT. min_stomate ) THEN 
                
                veget_max_new(i,j)= zero
                ind(i,j) = zero
                biomass(i,j,:,:) = zero
                PFTpresent(i,j) = .FALSE.
                senescence(i,j) = .FALSE.
                age(i,j) = zero
                when_growthinit(i,j) = undef
                everywhere(i,j) = zero
                carbon(i,:,j) = zero
                litter(i,:,j,:,:) = zero
                bm_to_litter(i,j,:,:) = zero
                turnover_daily(i,j,:,:) = zero
                
             ENDIF
             
          ENDIF ! End if PFT's coverage reduction
          
       ENDDO ! Loop over # PFTs
       
       !! 2.4 update 10 year-turnover pool content following flux emission
       !!     (linear decay (10%) of the initial carbon input)
       DO  l = 0, 8
          m = 10 - l
          cflux_prod10(i) =  cflux_prod10(i) + flux10(i,m)
          prod10(i,m)     =  prod10(i,m-1)   - flux10(i,m-1)
          flux10(i,m)     =  flux10(i,m-1)
          
          IF (prod10(i,m) .LT. 1.0) prod10(i,m) = zero
       ENDDO
       
       cflux_prod10(i) = cflux_prod10(i) + flux10(i,1) 
       flux10(i,1)     = 0.1 * prod10(i,0)
       prod10(i,1)     = prod10(i,0)
       
       !! 2.5 update 100 year-turnover pool content following flux emission\n
       DO   l = 0, 98
          m = 100 - l
          cflux_prod100(i)  =  cflux_prod100(i) + flux100(i,m)
          prod100(i,m)      =  prod100(i,m-1)   - flux100(i,m-1)
          flux100(i,m)      =  flux100(i,m-1)
          
          IF (prod100(i,m).LT.1.0) prod100(i,m) = zero
       ENDDO
       
       cflux_prod100(i)  = cflux_prod100(i) + flux100(i,1) 
       flux100(i,1)      = 0.01 * prod100(i,0)
       prod100(i,1)      = prod100(i,0)
       prod10(i,0)        = zero
       prod100(i,0)       = zero 
       
    ENDDO ! Loop over # pixels - domain size
    
  !! 3. history
    
    veget_max(:,:) = veget_max_new(:,:)
    convflux        = convflux/one_year*dt_days
    cflux_prod10    = cflux_prod10/one_year*dt_days
    cflux_prod100   = cflux_prod100/one_year*dt_days
    
    IF (bavard.GE.4) WRITE(numout,*) 'Leaving lcchange_main'
    
  END SUBROUTINE lcchange_main


! ================================================================================================================================
!! SUBROUTINE   : land_cover_change_main
!!
!>\BRIEF        Impact of land cover change on carbon stocks
!!
!! DESCRIPTION  : This subroutine is activated by setting LAND_COVER_CHANGE to  y 
!! in the run.def file. This sets the flag ::control%ok_land_cover_change.
!! In the case of 'LAND_COVER_CHANGE = y', the impact of land cover change on 
!! in-situ carbon stocks is computed in this subroutine. Fluxes from products are
!! now dealt with in a separate subroutine CHECK. The land cover change is read from
!! driver maps containing the new "maximal" coverage fraction of a PFT (::veget_max). 
!! On the basis of this difference, the amount of 'new establishment'/'biomass export',
!! and increase/decrease of each component, are estimated. The soil carbon from the
!! PFT which experienced a decrease in ::veget_max is pooled and used as the initial
!! soil carbon for the PFTs that experienced an increase in ::veget_max.\n
!!
!! RECENT CHANGE(S) : (1) Age-classes were introduced and (2) product use was moved 
!! to a separate module
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE land_cover_change_main (npts, dt_days, veget_max, veget, harvest_pool, &
                     harvest_type, harvest_cut, harvest_area, &
                     litter, carbon, lignin_struc, lignin_wood, &
                     biomass, PFTpresent, everywhere, when_growthinit, & 
                     leaf_frac, ind, circ_class_n, circ_class_biomass, &
                     co2_to_bm, forest_managed, &
                     KF, wstress_season, wstress_month, senescence, npp_longterm, &
                     age, lm_lastyearmax, harvest_pool_bound, resolution, &
                     bm_to_litter, leaf_age, circ_class_dist, orphan_flux, &
                     bm_sync, vir_wstress_fac, tau_eff_leaf, tau_eff_sap, &
                     tau_eff_root, veget_max_new, age_stand, last_cut, &
                     c_buried, mb_lcc_error, k_latosa_adapt)

    IMPLICIT NONE
    
 !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
    
    INTEGER, INTENT(in)                              :: npts                   !! Domain size - number of pixels (unitless)
    INTEGER(i_std), DIMENSION (:,:), INTENT(in)      :: forest_managed         !! forest management flag
    REAL(r_std), INTENT(in)                          :: dt_days                !! Time step of vegetation dynamics for stomate (days)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: circ_class_dist        !! The probability distribution of trees in a circ
                                                                               !! class in case of a redistribution (unitless).
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: veget                  !! Fraction of vegetation type including 
                                                                               !! non-biological fraction (unitless)   
    REAL(r_std), DIMENSION(npts,2), INTENT(in)       :: resolution             !! Resolution at each grid point (m) [1=E-W, 2=N-S]
    REAL(r_std), DIMENSION(:), INTENT(in)            :: harvest_pool_bound     !! The boundaries of the diameter classes
                                                                               !! in the wood harvest pools @tex $(m)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_root           !! Effective root turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_sap            !! Effective sapwood turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: tau_eff_leaf           !! Effective leaf turnover time that accounts
                                                                               !! waterstress (days)
    REAL(r_std), DIMENSION(:,:), INTENT(in)          :: vir_wstress_fac        !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm),INTENT(in)      :: veget_max_new          !! new "maximal" coverage fraction of a PFT on the  
                                                                               !! ground.  May sum to
                                                                               !! less than unity if the pixel has
                                                                               !! nobio area. (unitless, 0-1)

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts) ,INTENT(out)        :: mb_lcc_error            !! 


    !! 0.3 Modified variables   
    
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: PFTpresent             !! PFT present (0 or 1)
    LOGICAL, DIMENSION(:,:), INTENT(inout)           :: senescence             !! Flag for setting senescence stage (only 
                                                                               !! for deciduous trees)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: KF                     !! Scaling factor to convert sapwood mass into leaf 
                                                                               !! mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: k_latosa_adapt         !! Leaf to sapwood area adapted for long 
                                                                               !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: veget_max              !! "maximal" coverage fraction of a PFT on the ground
                                                                               !! May sum to
                                                                               !! less than unity if the pixel has
                                                                               !! nobio area. (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: co2_to_bm              !! CO2 taken from the atmosphere to get C to create  
                                                                               !! the seedlings @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: everywhere             !! is the PFT everywhere in the grid box or 
                                                                               !! very localized (after its introduction) (?)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: when_growthinit        !! how many days ago was the beginning of 
                                                                               !! the growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ind                    !! Density of individuals at the stand level 
                                                                               !! @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_season         !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: wstress_month          !! Water stress factor, based on hum_rel_daily
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: npp_longterm           !! "long term" net primary productivity
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: age                    !! mean age (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: lm_lastyearmax         !! last year's maximum leaf mass for each PFT 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_struc           !! ratio Lignine/Carbon in structural litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: lignin_wood            !! ratio Lignine/Carbon in woody litter,
                                                                               !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: carbon                 !! carbon pool: active, slow, or passive 
                                                                               !! @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: litter                 !! metabolic and structural litter, above and 
                                                                               !! below ground @tex ($gC m^{-2}$) @endtex 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n           !! Number of individuals in each circ class
                                                                               !! @tex $(ind m^{-2})$ @endtex   
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_frac              !! fraction of leaves in leaf age class (unitless;0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: leaf_age               !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: biomass                !! Stand level biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: bm_to_litter           !! Transfer of biomass to litter 
                                                                               !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass     !! Biomass components of the model tree within a 
                                                                               !! circumference class @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: harvest_pool           !! The wood and biomass that have been
                                                                               !! havested by humans @tex $(gC)$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_type           !! Type of management that resulted
                                                                               !! in the harvest (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_cut            !! Type of cutting that was used for the harvest
                                                                               !! (unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: harvest_area           !! Harvested area (m^{2})
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: orphan_flux            !! Storage for fluxes of PFTs that no longer exist. 
                                                                               !! Following the total destruction of a PFT in LCC 
                                                                               !! (veget_max_new = 0), a flux from before LCC needs a 
                                                                               !! storage @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:),INTENT(inout)      :: bm_sync                !! The difference betweeen the biomass in the 
                                                                               !! circ_classes and the total biomass 
                                                                               !! @tex $(gC m^{-2})$ @endtex
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(inout) :: age_stand            !! Age of the forest stand (years)  
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(inout) :: last_cut             !! Years since last thinning (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: c_buried               !! Carbon buried during a land cover change to
                                                                               !! a nobio land type (urban/ice).
                                                                               !!  @tex (gC dt^{-1})$ @endtex
    !! 0.4 Local variables


    REAL(r_std), DIMENSION(npts,nvm,norphans,nelements) &
                                                     :: orphan_flux_local      !! Storage for fluxes of PFTs that no longer exist. 
                                                                               !! Following the total destruction of a PFT in LCC 
                                                                               !! (veget_max_new = 0), a flux from before LCC needs a 
                                                                               !! storage @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    INTEGER(i_std)                                   :: ipts,ivm, imbc         !! Indices (unitless)
    INTEGER(i_std)                                   :: igroup,ifm             !! Indices (unitless)
    INTEGER(i_std)                                   :: idia,icir,ilage        !! Indices (unitless)
    INTEGER(i_std)                                   :: jcir,ilit,ilev         !! Indices (unitless)
    INTEGER(i_std)                                   :: ipar,iele,icarb        !! Indices (unitless)
    INTEGER(i_std)                                   :: ivma,iyoung,jvma       !! Indices (unitless)
    INTEGER(i_std)                                   :: nagec_temp             !! Temporary counter
    LOGICAL                                          :: lsync                  !! returns false if the biomass is not synced
    REAL(r_std)                                      :: total_losses           !! Sum of losses only in delta_veget (unitless, 0-1)
    REAL(r_std)                                      :: share_expanded         !! Share of the veget_max of the existing vegetation
                                                                               !! within a PFT over the total veget_max following 
                                                                               !! expansion of that PFT (unitless, 0-1)
    REAL(r_std), DIMENSION(npts)                     :: pixel_area             !! surface area of the pixel @tex ($m^{2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvmap)               :: veget_max_group        !! Total veget_max of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvmap)               :: veget_max_new_group    !! Total veget_max_new of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvmap)               :: delta_veget            !! Change in veget_max due to LCC restricted to the
                                                                               !! PFT groups thus not accounting for the age-classes
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: loss_gain              !! The same as delta_veget but distributed of all
                                                                               !! age classes and thus taking the age-classes into 
                                                                               !! account (unitless, 0-1)
    REAL(r_std), DIMENSION(npts)                     :: loss_gain_nobio        !! The change in nobio fraction during the LCC.  Right
                                                                               !! now we only take into account a single nobio type.
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(nlitt,nlevs,nelements)    :: litter_bank            !! Bank to store the litter that becomes available when
                                                                               !! part of a PFT is removed @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(ncarb)                    :: soil_bank              !! Bank to store soil carbon that becomes available when
                                                                               !! part of a PFT is removed  @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(nlevs)                    :: struct_ltr_bank        !! Bank to store the lignin in structural litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nlevs)                     :: woody_ltr_bank         !! Bank to store the lignin in woody litter that 
                                                                               !! becomes available when part of a PFT is removed 
                                                                               !! (0-1,unitless)
    REAL(r_std),DIMENSION(nvm,nparts,nelements)      :: fresh_litter           !! Pool of fresh litter that is being released during 
                                                                               !! LCC @tex ($gC m^{-2}$) @endtex 
    REAL(r_std),DIMENSION(nparts,nelements)          :: fresh_ltr_bank         !! Bank to store all the fresh litter from site 
                                                                               !! clearing during LCC. Weighted value of fresh_litter
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std)                                      :: moi_bank               !! bank to store moiavail_growingseason of the cleared
                                                                               !! PFT's (unitless; 0-1)
    REAL(r_std)                                      :: trees_needed           !! Number of trees leftthat should be present in a 
                                                                               !! circumference class when merging the available and 
                                                                               !! newly established vegetation
                                                                               !! @tex $(ind m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(ncirc)                    :: diameters_temp         !! Temporary variable to store the diameters of the
                                                                               !! different diameter classes (m)
    REAL(r_std), DIMENSION(npts,nvm,nparts,nelements):: new_biomass            !! Temporary variable for biomass 
                                                                               !! @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nleafages)       :: new_leaf_frac          !! Temporary variable for leaf_frac (unitless;0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_ind                !! Temporary variable for ind @tex $(m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_n       !! Temporary variable for circ_class_n of the 
                                                                               !! established vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ_class_n       !! Temporary variable for circ_class_n of the
                                                                               !! new vegetation
                                                                               !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: est_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)   :: tmp_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! established vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,ncirc,nparts,nelements) &
                                                     :: new_circ_class_biomass !! Temporary variable for circ_class_biomass of the
                                                                               !! new vegetation @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                    :: est_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(ncirc)                    :: tmp_circ_class_dia     !! Variable to store temporary values for 
                                                                               !! circ_class (m)
    REAL(r_std), DIMENSION(npts,nvm,ncirc)           :: new_circ               !! Temporary variable for circ (m)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_co2_to_bm          !! Temporary variable for co2_to_bm 
                                                                               !! @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_age                !! Temporary variable for age (years)
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_lm_lastyearmax     !! Temporary variable for lm_last_year 
                                                                               !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm)                 :: new_everywhere         !! Temporary variable for everywhere (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_when_growthinit    !! Dummy for when_growthinit (days)
    REAL(r_std), DIMENSION(npts,nvm)                 :: dum_npp_longterm       !! Dummy for npp_long_term
                                                                               !! @tex ($gC m^{-2} year^{-1}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements) &
                                                     :: check_intern           !! Contains the components of the internal
                                                                               !! mass balance chech for this routine
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: closure_intern         !! Check closure of internal mass balance
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_start             !! Start and end pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)       :: pool_end               !! Start and end pool of this routine 
                                                                               !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    LOGICAL, DIMENSION(npts,nvm)                     :: dum_PFTpresent         !! Dummy for PFTpresent (0 or 1)
    LOGICAL, DIMENSION(npts,nvm)                     :: dum_senescence         !! Dummy for senescence (true/false)
    REAL(r_std)                                      :: harvest_ratio_stem     !! The fraction of the reserve pools which are
                                                                               !! located in the stem (unitless, 0-1)
    REAL(r_std)                                      :: harvest_ratio_branch   !! The fraction of the reserve pools which are
                                                                               !! located in the branches/roots (unitless, 0-1)
    REAL(r_std)                                      :: harvest_ratio_litter   !! The fraction of the reserve pools which are
                                                                               !! moved to the litter (unitless, 0-1)
    REAL(r_std)                                      :: total_nonheart_biomass !! The total amount of non-heartwood biomass
                                                                               !! in a model tree
                                                                               !! @tex $(gC tree^{-1})$ @endtex
    REAL(r_std)                                      :: total_nonheart_biomass_stem !! The total amount of non-heartwood biomass
                                                                               !! in a model tree stem
                                                                               !! @tex $(gC tree^{-1})$ @endtex
    REAL(r_std)                                      :: total_nonheart_biomass_branch !! The total amount of non-heartwood biomass
                                                                               !! in a model tree branches/roots
                                                                               !! @tex $(gC tree^{-1})$ @endtex
    REAL(r_std) :: temp_sum,temp_total
    REAL(r_std), DIMENSION(npts)                     :: change_bio             !! The overall change in vegetative fraction for
                                                                               !! this pixel (0-1, unitless)
    REAL(r_std), DIMENSION(npts)                     :: change_nobio           !! The overall change in non-vegetative fraction for
                                                                               !! this pixel (0-1, unitless)
    REAL(r_std), DIMENSION(nlitt,nlevs)              :: litter_weight_expanded !! The fraction of litter on the expanded
                                                                               !! PFT.
                                                                               !! @tex $-$ @endtex
    REAL(r_std), DIMENSION(nlevs)                    :: litter_weight_struc    !! A weighting factor for the structural litter.
                                                                               !! (unitless)
    REAL(r_std), DIMENSION(nlevs)                    :: litter_weight_woody    !! A weighting factor for the woody litter.
                                                                               !! (unitless)
    LOGICAL                                          :: lconverged             !! A flag for loop convergence
    INTEGER(i_std)                                   :: max_pft                !! The number of the PFT with the largest addition
                                                                               !! in land area.
    INTEGER(i_std)                                   :: min_pft                !! The number of the PFT with the largest reducation
                                                                               !! in land area (most negative value).
    INTEGER(i_std)                                   :: nsteps                 !! The number of convergence steps tried.
    INTEGER(i_std)                                   :: ncount                 !! The number of PFTs with significant changes
                                                                               !! in vegetation.
    INTEGER(i_std)                                   :: nagec_local            !! The number of age classes in this map PFT.
    REAL(r_std)                                      :: difference             !! How much leftover veget_max we have
    REAL(r_std)                                      :: temp_sum_old           !! 
    REAL(r_std)                                      :: temp_sum_new           !! 
    REAL(r_std), DIMENSION(npts,nvm)                 :: veget_max_new_adj      !! new "maximal" coverage fraction of a PFT on the  
                                                                               !! ground adjusted for numerical precision.  
                                                                               !! May sum to
                                                                               !! less than unity if the pixel has
                                                                               !! nobio area. (unitless, 0-1)
    !+++ TEMP++++
    REAL(r_std), DIMENSION(npts,nnobio)    ::  frac_nobio_start,frac_nobio_end

!_ ================================================================================================================================

    !+++++++ TEMP +++++
    frac_nobio_start(:,:)=zero
    frac_nobio_end(:,:)=zero
    DO ipts=1,npts
       frac_nobio_start(ipts,1)=un-SUM(veget_max(ipts,:))
       IF(frac_nobio_start(ipts,1) .GT. un)THEN
          WRITE(numout,*) 'ERROR: nobio fraction greater than one in LCC!'
          WRITE(numout,*) 'ipts, frac_nobio_start(ipts,1): ',ipts, frac_nobio_start(ipts,1)
          DO ivm=1,nvm
             WRITE(numout,*) ivm,veget_max(ipts,ivm)
          ENDDO
          STOP
       ENDIF
    ENDDO
    !+++++++++++++++++++

    !+++++++++++ TEMP

!!$    veget_max_new_adj(1,1)=0.07098031
!!$    veget_max_new_adj(1,2)=0.00000000
!!$    veget_max_new_adj(1,3)=0.00000000
!!$    veget_max_new_adj(1,4)=0.00000000
!!$    veget_max_new_adj(1,5)=0.00023827
!!$    veget_max_new_adj(1,6)=0.00000000
!!$    veget_max_new_adj(1,7)= 0.00000000
!!$    veget_max_new_adj(1,8)=  0.00000000
!!$    veget_max_new_adj(1,9)= 0.00000
!!$    veget_max_new_adj(1,10)=  0.00000000
!!$    veget_max_new_adj(1,11)= 0.00000000
!!$    veget_max_new_adj(1,12)= 0.00000000
!!$    veget_max_new_adj(1,13)= 0.00196709
!!$    veget_max_new_adj(1,14)= 0.00000000
!!$    veget_max_new_adj(1,15)=  0.00000000
!!$    veget_max_new_adj(1,16)=  0.00000000
!!$    veget_max_new_adj(1,17)=  0.00040394
!!$    veget_max_new_adj(1,18)=  0.00000000
!!$    veget_max_new_adj(1,19)=   0.00000000
!!$    veget_max_new_adj(1,20)=  0.00000000
!!$    veget_max_new_adj(1,21)=   0.00000000
!!$    veget_max_new_adj(1,22)=  0.00000000
!!$    veget_max_new_adj(1,23)=  0.00000000
!!$    veget_max_new_adj(1,24)=  0.00000000
!!$    veget_max_new_adj(1,25)=  0.00000000
!!$    veget_max_new_adj(1,26)=   0.00000000
!!$    veget_max_new_adj(1,27)=   0.00000000
!!$    veget_max_new_adj(1,28)=   0.00000000
!!$    veget_max_new_adj(1,29)=  0.00000000
!!$    veget_max_new_adj(1,30)=  0.00942447
!!$    veget_max_new_adj(1,31)=   0.00000000
!!$    veget_max_new_adj(1,32)=   0.00000000
!!$    veget_max_new_adj(1,33)=  0.00000000
!!$    veget_max_new_adj(1,34)=  0.04421792
!!$    veget_max_new_adj(1,35)=  0.00000000
!!$    veget_max_new_adj(1,36)=   0.00000000
!!$    veget_max_new_adj(1,37)=   0.00000000
!!$    veget_max_new_adj(1,38)=  0.04851483
!!$    veget_max_new_adj(1,39)=   0.00000000
!!$    veget_max_new_adj(1,40)=   0.00000000
!!$    veget_max_new_adj(1,41)=  0.00000000
!!$    veget_max_new_adj(1,42)=  0.00000000
!!$    veget_max_new_adj(1,43)=  0.05385288
!!$    veget_max_new_adj(1,44)=  0.00000000
!!$    veget_max_new_adj(1,45)=  0.00000000
!!$    veget_max_new_adj(1,46)=   0.00000000
!!$    veget_max_new_adj(1,47)=  0.00000000
!!$    veget_max_new_adj(1,48)=  0.00000000
!!$    veget_max_new_adj(1,49)=  0.00000000
!!$    veget_max_new_adj(1,50)=   0.00000000
!!$    veget_max_new_adj(1,51)=  0.00000000
!!$    veget_max_new_adj(1,52)=  0.00000000
!!$    veget_max_new_adj(1,53)=  0.00000000
!!$    veget_max_new_adj(1,54)=  0.00000000
!!$    veget_max_new_adj(1,55)=  0.00000000
!!$    veget_max_new_adj(1,56)=  0.00000000
!!$    veget_max_new_adj(1,57)=  0.00000000
!!$    veget_max_new_adj(1,58)=  0.00000000
!!$    veget_max_new_adj(1,59)=  0.00000000
!!$    veget_max_new_adj(1,60)=  0.00000000
!!$    veget_max_new_adj(1,61)=  0.00000000
!!$    veget_max_new_adj(1,62)=  0.35551621
!!$    veget_max_new_adj(1,63)=  0.00000000
!!$    veget_max_new_adj(1,64)=  0.41488408
!!$    veget_max_new_adj(1,65)=  0.00000000

!!$    veget_max_new_adj(1,1)=0.
!!$    veget_max_new_adj(1,2)=0.00000000
!!$    veget_max_new_adj(1,3)=0.00000000
!!$    veget_max_new_adj(1,4)=0.00000000
!!$    veget_max_new_adj(1,5)=0.5
!!$    veget_max_new_adj(1,6)=0.00000000
!!$    veget_max_new_adj(1,7)= 0.00000000
!!$    veget_max_new_adj(1,8)=  0.00000000
!!$    veget_max_new_adj(1,9)= 0.00
!!$    veget_max_new_adj(1,10)=  0.00000000
!!$    veget_max_new_adj(1,11)= 0.00000000
!!$    veget_max_new_adj(1,12)= 0.00000000
!!$    veget_max_new_adj(1,13)= 0.0
!!$    veget_max_new_adj(1,14)= 0.00000000
!!$    veget_max_new_adj(1,15)=  0.00000000
!!$    veget_max_new_adj(1,16)=  0.00000000
!!$    veget_max_new_adj(1,17)=  0.0
!!$    veget_max_new_adj(1,18)=  0.00000000
!!$    veget_max_new_adj(1,19)=   0.00000000
!!$    veget_max_new_adj(1,20)=  0.00000000
!!$    veget_max_new_adj(1,21)=   0.00000000
!!$    veget_max_new_adj(1,22)=  0.00000000
!!$    veget_max_new_adj(1,23)=  0.00000000
!!$    veget_max_new_adj(1,24)=  0.00000000
!!$    veget_max_new_adj(1,25)=  0.00
!!$    veget_max_new_adj(1,26)=   0.00000000
!!$    veget_max_new_adj(1,27)=   0.00000000
!!$    veget_max_new_adj(1,28)=   0.00000000
!!$    veget_max_new_adj(1,29)=  0.00000000
!!$    veget_max_new_adj(1,30)=   0.00
!!$    veget_max_new_adj(1,31)=   0.00000000
!!$    veget_max_new_adj(1,32)=   0.00000000
!!$    veget_max_new_adj(1,33)=  0.00000000
!!$    veget_max_new_adj(1,34)=   0.0
!!$    veget_max_new_adj(1,35)=  0.00000000
!!$    veget_max_new_adj(1,36)=   0.00000000
!!$    veget_max_new_adj(1,37)=   0.00000000
!!$    veget_max_new_adj(1,38)=  0.0
!!$    veget_max_new_adj(1,39)=   0.00000000
!!$    veget_max_new_adj(1,40)=   0.00000000
!!$    veget_max_new_adj(1,41)=  0.00000000
!!$    veget_max_new_adj(1,42)=  0.00000000
!!$    veget_max_new_adj(1,43)=   0.0
!!$    veget_max_new_adj(1,44)=  0.00000000
!!$    veget_max_new_adj(1,45)=  0.00000000
!!$    veget_max_new_adj(1,46)=   0.00000000
!!$    veget_max_new_adj(1,47)=  0.00000000
!!$    veget_max_new_adj(1,48)=  0.00000000
!!$    veget_max_new_adj(1,49)=  0.00000000
!!$    veget_max_new_adj(1,50)=   0.00000000
!!$    veget_max_new_adj(1,51)=  0.00000000
!!$    veget_max_new_adj(1,52)=  0.00000000
!!$    veget_max_new_adj(1,53)=  0.00000000
!!$    veget_max_new_adj(1,54)=  0.00000000
!!$    veget_max_new_adj(1,55)=  0.00000000
!!$    veget_max_new_adj(1,56)=  0.00000000
!!$    veget_max_new_adj(1,57)=  0.00000000
!!$    veget_max_new_adj(1,58)=  0.00000000
!!$    veget_max_new_adj(1,59)=  0.00000000
!!$    veget_max_new_adj(1,60)=  0.00000000
!!$    veget_max_new_adj(1,61)=  0.00000000
!!$    veget_max_new_adj(1,62)=  0.5
!!$    veget_max_new_adj(1,63)=  0.00000000
!!$    veget_max_new_adj(1,64)=  0.
!!$    veget_max_new_adj(1,65)=  0.00000000
    !+++++++++++++++++++++

    IF (bavard.GE.3) WRITE(numout,*) 'Entering land cover change main'

    ! Sometimes maps are read in which are created in single
    ! precision.  Values in the code may be double precision,
    ! however.  min_stomate is usually set to 1e-8, which is
    ! similar to single precision.  Depending on the maps
    ! used, this can frequently cause mass balance errors
    ! on the order of 1e-5 to 1e-8 as the old veget_max
    ! and the new veget max sum to a value just less
    ! than min_stomate, which when multiplied by the pools
    ! used gives a value greater than min_stomate of carbon
    ! which is considered lost.  This first loop corrects for
    ! that and closes the mass balance.
    DO ipts=1,npts
       temp_sum_new=zero
       temp_sum_old=zero
       DO ivm=1,nvm
          temp_sum_new=temp_sum_new+veget_max_new(ipts,ivm)
          temp_sum_old=temp_sum_old+veget_max(ipts,ivm)
       ENDDO
       IF(ld_lcc .AND. ipts == test_grid)THEN
          WRITE(numout,*) 'temp_sum_old: ',temp_sum_old
          WRITE(numout,*) 'temp_sum_new: ',temp_sum_new
       ENDIF
       difference=ABS(temp_sum_old-temp_sum_new)
       ! If the difference is too big, there is a significant difference
       ! in the nobio fraction, which has to be specially taken
       ! care of.  We let a check below deal with that.
       IF(difference .NE. zero .AND. difference .LT. min_stomate)THEN
          DO ivm=1,nvm
             veget_max_new_adj(ipts,ivm)=veget_max_new(ipts,ivm)*temp_sum_old/temp_sum_new
          ENDDO
       ELSE
          ! Nothing to change.
          DO ivm=1,nvm
             veget_max_new_adj(ipts,ivm)=veget_max_new(ipts,ivm)
          ENDDO
       ENDIF
    ENDDO
    
    !  +++COMMENT+++
    !  This part is poorly coded as it uses the externalisation to
    !  implement the age classes. The advanatage of this approach is
    !  that is rather simple to implement. The alternative requires
    !  that a dimension for age classes is added to almost all fluxes
    !  and pool in sechiba and stomate. Also the externalisation would
    !  need to be adjusted as the parameters are now be used for the
    !  different age classes.
    !
    !  The coding is considered poor because it is : (1) sensitive to 
    !  order in which the PFTs are defined and (2) inconsistent as
    !  cohorts (within a stand got an own dimension) and age classes
    !  (across stands) are implemented through the externalisation.
    !  An example for 2 age classes and 8 tree PFTs
    !  PFT_1: bare soil
    !  PFT_2: PFT2_1 age class 1 (youngest)
    !  PFT_3: PFT2_2 age class 2 (youngest + 1)
    !  PFT_4: PFT3_1
    !  PFT_5: PFT3_2
    !  ...
    !  PFT_18: PFT 10 agricultural PFTs don't have age classes
    !  PFT_19: PFT 11 agricultural PFTs don't have age classes
    !  PFT_20: PFT 11 agricultural PFTs don't have age classes
    !  PFT_21: PFT 11 agricultural PFTs don't have age classes
    !  
    !  A PFT group are all age classes a a certain PFT/species
    !
    !  Some of this complexity could be avoided by preparing the maps
    !  for the exact the number of age classes we want to deal with
    !  but that just shifts the complexity. It also means that we have
    !  to prescribe the age classes, which is not our goal. 
    !  For now I assumed the maps only consider the PFTs without age classes.
    !  +++++++++++
   
    DO ipts=1,npts
       DO ivm=1,nvm
          IF(ipts == test_grid .AND. ivm == test_pft)THEN
             WRITE(numout,*) 'orphan_flx at the beginning ',orphan_flux(ipts,ivm,:,:)
          ENDIF
       ENDDO
    ENDDO
     
 !! 1. Initialization
    
    !! 1.1 Initialize variables
    delta_veget(:,:) = zero 
    loss_gain(:,:) = zero
    veget_max_group(:,:) = zero
    veget_max_new_group(:,:) = zero
    ! There is a chance that the orphan fluxes will already have a value
    ! at this point.  Store them in the local array for the calculation
    ! of the subroutine.
    ! There is also the chance that co2_to_bm will have something in it, for
    ! example, if something was prescribed in this step.  We we do not save
    ! that as an orphan flux, it will not be accounted for in the pool_end,
    ! which means we get a mass balance error even if no LCC happens in this
    ! step.
    orphan_flux_local(:,:,:,:)=zero
    !DO ipts=1,npts
    !   DO ivm=1,nvm
    !      IF(co2_to_bm(ipts,ivm) .GT. min_stomate)THEN
    !         orphan_flux_local(ipts,ivm,ico2bm,icarbon) = co2_to_bm(ipts,ivm)
    !      END IF
    !   ENDDO
    !ENDDO

    ! Surface area (m2) of the pixel
    pixel_area(:) = resolution(:,1) * resolution(:,2)

    !! 1.2 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 6. Initial biomass and harvest pool all other
    !  relevant pools were just set to zero.
    pool_start(:,:,:) = zero
    DO iele = 1,nelements
       
       IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Start pool 1 ',pool_start(test_grid,test_pft,iele)
          ENDIF
       ENDIF

       ! co2_to_bm
       ! We subtract any old flux here.  The reason is that this old flux will
       ! not be accounted for at the end when the final pools are calculated,
       ! so if we don't remove it now we get a mass imbalance.
       pool_start(:,:,iele) = pool_start(:,:,iele) + co2_to_bm(:,:) &
            - orphan_flux(:,:,ico2bm,icarbon) 

       IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Start pool 2 ',pool_start(test_grid,test_pft,iele),&
                  orphan_flux(test_grid,test_pft,ico2bm,icarbon),&
                  orphan_flux(test_grid,test_pft,ivegold,icarbon),&
                  co2_to_bm(test_grid,test_pft)
          ENDIF
       ENDIF

       ! Biomass pool + bm_to_litter
       DO ipar = 1,nparts
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) + bm_to_litter(:,:,ipar,iele)) * &
               veget_max(:,:)
          IF(test_grid > 0)THEN
             IF(test_pft < nvm)THEN
                WRITE(numout,'(A,I5,10ES20.10)') 'Start biomass ',ipar,biomass(test_grid,test_pft,ipar,iele) , &
                     bm_to_litter(test_grid,test_pft,ipar,iele), &
                     veget_max(test_grid,test_pft)
             ENDIF
          ENDIF
       ENDDO

      IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Start pool 3 ',pool_start(test_grid,test_pft,iele)
          ENDIF
       ENDIF

       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilit = 1,nlitt
          DO ilev = 1,nlevs
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  litter(:,ilit,:,ilev,iele) * veget_max(:,:)
          ENDDO
       ENDDO

      IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Start pool 4 ',pool_start(test_grid,test_pft,iele)
          ENDIF
       ENDIF

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO

      IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Start pool 5 ',pool_start(test_grid,test_pft,iele)
          ENDIF
       ENDIF

    ENDDO

    ! The biomass harvest pool is expressed in gC pixel-1 So, it 
    ! shouldn't be multiplied by veget_max but it should be divided
    ! by pixel_area to obtain gC m-2.
    DO ivm = 1,nvm
       pool_start(:,ivm,icarbon) = pool_start(:,ivm,icarbon) + &
            SUM(harvest_pool(:,ivm,:,icarbon),2) / &
            pixel_area(:)
    ENDDO
    
!! 2. Calculate common variables 
 
    !  For several calculations the total cover of all age class within 
    !  its group (::veget_max_group) is required.  We also need the
    !  vegetation fraction of the PFT map arranged into groups.  Notice
    !  that this number is already given in terms of groups, but it is
    !  stored on a grid using all nvm pfts, i.e. only nvmap values of
    !  veget_max_new(ipts,:) are non-zero.
    
    DO ipts = 1,npts
 
       DO ivm=1,nvm

          igroup=agec_group(ivm)
          
          veget_max_group(ipts,igroup) = veget_max_group(ipts,igroup) + &
                        veget_max(ipts,ivm)
          veget_max_new_group(ipts,igroup) = veget_max_new_group(ipts,igroup) + &
                        veget_max_new_adj(ipts,ivm)

       ENDDO ! loop over PFTs

    ENDDO ! loop over #pixels - domain size

    IF (ld_lcc) THEN
       WRITE(numout,*) 'Finished computing veget_max_group.'
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,1)
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,2)
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,5)
    ENDIF


 !! 3. Check consistency of the LCC maps

    ! Within each pixel the fraction of losses should equal the fraction of gains.
    ! The soil carbon avaible from the losses is averaged and used as the initial 
    ! soil carbon for the gains. This assumption is acceptable when the fraction of 
    ! none biological land uses remains constant. If this fraction is changing i.e.
    ! following glacier retreat, the current approach will need to be checked and
    ! most likely adjusted. The same applies in case vegetation is removed for
    ! urban sprawing. This routine will need to decide whether the soil carbon is 
    ! simply buried under the hard surfaces or whether it decomposes?

    !! 3.1 Calculate the change in vegetation cover
    delta_veget(:,:) =  veget_max_new_group(:,:) - veget_max_group(:,:)

    !---TEMP---
    IF (ld_lcc) THEN
       WRITE(numout,*) 'Before normalization of delta_veget'
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,3)
    ENDIF
    !----------

    !! 3.2 Readjust the change in vegetation cover for the PFTs so that
    !!     no change is less than min_vegfrac and that no change leads
    !!     to PFTs values which are too high or too low.
    CALL adjust_delta_veget(npts,veget_max,veget_max_group,veget_max_new_adj,&
       veget_max_new_group,loss_gain,delta_veget)

   !---TEMP---
    IF (ld_lcc) THEN
       WRITE(numout,*) 'After normalization of delta_veget and loss_gain'
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,3)
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,4)
    ENDIF
    !----------

    ! Check whether gains equal losses. If not it is unlikely that the C-balance
    ! will be closed.
    DO ipts = 1,npts
       change_bio(ipts)=SUM(delta_veget(ipts,:))
       ! The nobio fractions are the leftover fractions when all the vegetation
       ! fractions are summed up.
       change_nobio(ipts)=(un-SUM(veget_max_new_group(ipts,:)))-&
            (un-SUM(veget_max_group(ipts,:)))
       IF ( ABS(change_bio(ipts)+change_nobio(ipts)) .GE. min_stomate) THEN
          ! Losses and gains don't cancel each other out
          WRITE(numout,*) "Error: LCC losses and gains don't cancel each other out in pixel, ", ipts
          WRITE(numout,*) 'Mismatch: ',change_bio(ipts)+change_nobio(ipts)
          WRITE(numout,*) 'change_bio: ',change_bio(ipts)
          WRITE(numout,*) 'change_nobio: ',change_nobio(ipts)
          WRITE(numout,*) 'sum of delta_veget: ',SUM(delta_veget(ipts,:))
          WRITE(numout,*) 'sum of veget_max_new_group: ',SUM(veget_max_new_group(ipts,:))
          WRITE(numout,*) 'sum of veget_max_new_adj: ',SUM(veget_max_new_adj(ipts,:))
          WRITE(numout,*) 'sum of veget_max_group: ',SUM(veget_max_group(ipts,:))
          CALL ipslerr_p (2,'land_cover_main', 'Losses and gains do not cancel in land_cover_change.',&
               'Look for Error in out_orchidee file for details','')

       ELSE

          ! Losses and gains cancel each other out
          !---TEMP---
          IF (ld_lcc .AND. test_grid == ipts) THEN
             WRITE(numout,*) "LCC losses and gains cancel each other out "
             WRITE(numout,*) 'OK: ',change_bio(ipts),change_nobio(ipts)
          ENDIF
          !----------

       ENDIF

       ! Right now, we are only confident are accounting of nobio fraction
       ! is reasonable if the changes are really, really small (basically
       ! numerical issues).  If this is not the case, print a warning,
       ! since someone should think this through more clearly.  This check should
       ! be taken care of already in slowproc_update.
       IF(ABS(change_nobio(ipts)) .GT. min_vegfrac)THEN
          WRITE(numout,*) 'LCC1 WARNING: Large change in nobio fraction in LCC!'
          WRITE(numout,*) 'ipts: ',ipts
          WRITE(numout,*) 'change_nobio: ',change_nobio(ipts)
          WRITE(numout,*) 'ivm veget_max_new_adj veget_max'
          DO ivm=1,nvm
             WRITE(numout,'(I7,2F12.8)') ivm,veget_max_new_adj(ipts,ivm),veget_max(ipts,ivm)
          ENDDO
          CALL ipslerr_p (2,'land_cover_main', 'Large change in nobio fraction',&
               'Look for LCC1 WARNING in out_orchidee file for details','')
       ENDIF

    ENDDO ! npts

 !! 5. Calculate in-situ land cover changes

    !  NOTE: within the model framework there is large conceptual
    !  difference between management and clear cuts on the one
    !  hand and LCC on the other hand. Within the model LCC
    !  DOES NOT change the biomass, ind, ... of the PFTs that
    !  experience a loss (because we store those in gC m-2). LCC 
    !  only changes the values of veget_max for those PFTs. For
    !  this reason it was decided not to make use of lpj_kill.f90
    !  and sapiens_gap.f90 (as is done for forest management) 
    !  because this would be in conflict with the meaning of the 
    !  variable ::circ_class_kill. It is not impossible to work 
    !  around this issues but it could result in difficulties with
    !  the order of code in more complex cases were part of the PFT 
    !  experiences a surface loss and at the same time is ready to
    !  get thinned or clear cut. It appears cleaner to deal with
    !  LCC up front in a self-contained module.

    !! 5.1 Land cover losses within a pixel
    !  The losses need to be calculated first such that the soil
    !  carbon (and other elements) can be put in the soil&litter-bank. 
    !  Subsequently, the soil&litter-bank is used to initialize the 
    !  soil carbon of the new PFTs. The biomass is moved into a 
    !  harvest pool.
    !  NOTE: make sure that all variables for PFT1 are initialized!
    !  If not LCC won't properly work.
    DO ipts = 1, npts ! Loop over # pixels - domain size    

       !! 5.1.1 Initialize the soil&litter-bank
       !  NOTE Some of these variables have a dimension for ::nelements
       !  others don't. Make sure ::nelements is accounted for when 
       !  N is added to the model.
       soil_bank(:) = zero
       litter_bank(:,:,:) = zero
       struct_ltr_bank(:) = zero
       woody_ltr_bank(:) = zero
       fresh_litter(:,:,:) = zero
       fresh_ltr_bank(:,:) = zero
       moi_bank = zero
        
       !! 5.1.2 Loss in PFT cover, update harvest and soil&litter-C-bank

       !  To calculate the relative change in cover fraction, PFT-specific 
       !  changes need to be normalized by the total change which is in 
       !  this case the total loss. Total losses includes losses in PFT1
       total_losses = SUM(loss_gain(ipts,:), MASK = loss_gain(ipts,:) .LT. zero)

       ! Calculate the factor to weight the lignin ratios by when merging.
       ! This depends on the amount of litter on the PFTs where vegetation
       ! is being cut.
       litter_weight_struc(:)=zero
       litter_weight_woody(:)=zero
       DO ivm = 1, nvm
          IF ( loss_gain(ipts,ivm) .LE. -min_stomate ) THEN
             DO ilev=1,nlevs
                litter_weight_struc(ilev)=litter_weight_struc(ilev)+&
                     litter(ipts,istructural,ivm,ilev,icarbon) * ABS(loss_gain(ipts,ivm))
                litter_weight_woody(ilev)=litter_weight_woody(ilev)+&
                     litter(ipts,iwoody,ivm,ilev,icarbon) * ABS(loss_gain(ipts,ivm))
             ENDDO
          ENDIF
       ENDDO

       DO ivm = 1, nvm

          ! The PFT cover decreases, move wood into the harvest pool, the 
          ! stumps and branches are supposed to be exported from the site 
          ! so they go into the harvest pool as well but a separated pool
          ! was introduced for separate treatment in the product use module
          ! Leaves, fine roots and fruits are deposited on the soil&litter
          ! bank and will be attributed to the new PFT. This order of 
          ! calculations ensures that the litter ends up in the right PFT
          ! and that it is only decomposed once.         
          IF ( loss_gain(ipts,ivm) .LE. -min_stomate ) THEN

             !! 5.1.2.1 Biomass losses of trees
             IF (is_tree(ivm)) THEN

                ! We will need the trunk diameters for all the circ classes in 
                ! order to figure out which harvest pool to put them in.
                diameters_temp(:) = &
                     wood_to_dia(circ_class_biomass(ipts,ivm,:,:,icarbon),ivm)

                ! Store the management stategy of the pixel and PFT
                ifm = forest_managed(ipts,ivm)

                ! Now we have the stems diameters.  These are what we want to
                ! save, and we are going to classify them by PFT type, location, 
                ! and diameter so that we can compare to FAO statistics and do 
                ! some more interesting post-treatment (sapiens_product_use.f90).
                DO icir = 1,ncirc

                   ! We want to make sure we divide up the labile and carbohydrate
                   ! reserve pools in the same was as in other management.  We
                   ! assume that when a tree is cut down, it cannot rearrange 
                   ! these pools, and therefore the amount that is harvested
                   ! is the ratio that is stored in the harvest wood.  We assume
                   ! that the reserve pools are stored equally in all non-heart
                   ! tissue.  The harvest right now goes into three different
                   ! places: two harvest pools and litter.  We need the fractions
                   ! for the reserve pools going into each place.
                   total_nonheart_biomass = &
                        circ_class_biomass(ipts,ivm,icir,isapbelow,icarbon)+&
                        circ_class_biomass(ipts,ivm,icir,iroot,icarbon)+&
                        circ_class_biomass(ipts,ivm,icir,ileaf,icarbon)+&
                        circ_class_biomass(ipts,ivm,icir,ifruit,icarbon)+&
                        circ_class_biomass(ipts,ivm,icir,isapabove,icarbon)
                   ! This includes the roots and the branches, since it goes into
                   ! a different pool.
                   total_nonheart_biomass_branch=branch_ratio(ivm)*&
                        circ_class_biomass(ipts,ivm,icir,isapabove,icarbon)+&
                        circ_class_biomass(ipts,ivm,icir,isapbelow,icarbon)
                   total_nonheart_biomass_stem=(un-branch_ratio(ivm))*&
                        circ_class_biomass(ipts,ivm,icir,isapabove,icarbon)

                   ! These are the three ratios we need to move the reserve pools around.
                   harvest_ratio_branch=total_nonheart_biomass_branch/total_nonheart_biomass
                   harvest_ratio_stem=total_nonheart_biomass_stem/total_nonheart_biomass
                   harvest_ratio_litter=un - harvest_ratio_stem -  harvest_ratio_branch

                   dia_loop:DO idia=1,ndia_harvest

                      IF(diameters_temp(icir) .LE. harvest_pool_bound(idia))THEN

                         ! This is the correct diameter class
                         ! Move wood (excluding the branches) and the carbres and labile
                         ! C contained in the wood into the harvest pool. Note that the 
                         ! unit of circ_class_biomass is gC tree-1, so multiply with the 
                         ! number of trees to obtain gC m-2. To avoid complexities when 
                         ! dealing with the product pool multiply with the cover change 
                         ! and the surface area of the pixel to obtain a weighted mean 
                         ! expressed in gC.
                         harvest_pool(ipts,ivm,idia,icarbon) = &
                              harvest_pool(ipts,ivm,idia,icarbon) + &  
                              ABS(loss_gain(ipts,ivm)) * pixel_area(ipts) * & 
                              ( (un - branch_ratio(ivm)) * circ_class_n(ipts,ivm,icir) * &
                              ( circ_class_biomass(ipts,ivm,icir,isapabove,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,iheartabove,icarbon)) +&
                              ! reserves
                              harvest_ratio_stem * circ_class_n(ipts,ivm,icir) * &
                              (circ_class_biomass(ipts,ivm,icir,icarbres,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,ilabile,icarbon) ) )

                         ! Associated harvest variables
                         harvest_type(ipts,ivm) = ifm
                         harvest_cut(ipts,ivm) = icut_lcc_wood
                         harvest_area(ipts,ivm) = ABS(loss_gain(ipts,ivm) * pixel_area(ipts))

                         ! Move stumps and branches and the carbres and labile C 
                         ! contained in them into a separate harvest pool. Stumps 
                         ! and branches are removed from the site but can now be 
                         ! treated separately from stem wood in the product use 
                         ! module (sapiens_product_use.f90)       
                         harvest_pool(ipts,ivm,idia,icarbon) = &
                              harvest_pool(ipts,ivm,idia,icarbon) + &
                              ABS(loss_gain(ipts,ivm)) * pixel_area(ipts) * &
                              ! branches
                              ( branch_ratio(ivm) * circ_class_n(ipts,ivm,icir) * &
                              ( circ_class_biomass(ipts,ivm,icir,isapabove,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,iheartabove,icarbon)) + &
                              ! reserves
                              harvest_ratio_branch * circ_class_n(ipts,ivm,icir) * &
                              (circ_class_biomass(ipts,ivm,icir,icarbres,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,ilabile,icarbon) )  + &
                              ! stumps (all icarbres and ilabile is distributed
                              ! over the stems and branches)
                              circ_class_n( ipts,ivm,icir) * &
                              ( circ_class_biomass(ipts,ivm,icir,isapbelow,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,iheartbelow,icarbon) ) )
                         
                         ! Associated harvest variables
                         harvest_type(ipts,ivm) = ifm
                         harvest_cut(ipts,ivm) = icut_lcc_res
                         harvest_area(ipts,ivm) = ABS(loss_gain(ipts,ivm) * pixel_area(ipts))
                        
                         ! Deposit the leaves, roots and fruit into a temporary litter 
                         ! pool and place it in the soil and litter bank
                         fresh_litter(ivm,ileaf,icarbon) = &
                              fresh_litter(ivm,ileaf,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,ileaf,icarbon) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,iroot,icarbon) = &
                              fresh_litter(ivm,iroot,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,iroot,icarbon) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,ifruit,icarbon) = &
                              fresh_litter(ivm,ifruit,icarbon) + &
                              circ_class_biomass(ipts,ivm,icir,ifruit,icarbon) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,icarbres,icarbon) = &
                              fresh_litter(ivm,icarbres,icarbon) + &
                              harvest_ratio_litter*&
                              circ_class_biomass(ipts,ivm,icir,icarbres,icarbon) * &
                              circ_class_n(ipts,ivm,icir)
                         fresh_litter(ivm,ilabile,icarbon) = &
                              fresh_litter(ivm,ilabile,icarbon) + &
                              harvest_ratio_litter*&
                              circ_class_biomass(ipts,ivm,icir,ilabile,icarbon) * &
                              circ_class_n(ipts,ivm,icir)

                         EXIT dia_loop

                      ENDIF ! correct diameter class

                      ! If we are here, it means the diameter of this trunk is larger 
                      ! than the upper boundary of our harvest bins.  Considering the 
                      ! upper boundary is set to be 99999 meters in constants.f90, 
                      ! something has gone wrong!
                      IF(idia == ndia_harvest)THEN

                         WRITE(numout,*) 'Our tree is too big to fit into the harvest pools.'
                         WRITE(numout,*) 'ipts, ivm, icir: ',ipts, ivm, icir
                         WRITE(numout,*) 'diameters_temp(icir), harvest_pool_bound(idia)',&
                              diameters_temp(icir),harvest_pool_bound(idia)
                         CALL ipslerr_p (3,'land_cover_main', 'harvest pool error (see output file)','','')

                      ENDIF

                   ENDDO dia_loop

                ENDDO ! ncirc

             !! 5.1.2.2 Biomass losses of grasses and crops
             !  During LCC grasses and crops are simply plowed under 
             !  and should thus be move into ::fresh_litter which is
             !  moved later on into the litter bank ::fresh_ltr_bank
             !  and finally into ::bm_to_litter. The bank is needed
             !  to take a weighted average.
             ELSE

                fresh_litter(ivm,:,icarbon) = fresh_litter(ivm,:,icarbon) + &
                     biomass(ipts,ivm,:,icarbon)
                
             ENDIF ! is_tree(ivm)

             !! 5.1.2.3 Deposit soil and litter and soil&litter bank
             ! Deposit the released soil&litter pools on the soil&litter-bank. 
             ! The units of the C-pools are in gC m-2, hence, they need to be 
             ! weighted by their relative change in cover fraction to obtain 
             ! the C-pool (gC pixel-2) of the soil&litter-C-bank.     
             litter_bank(:,:,:) = litter_bank(:,:,:) + &
                  loss_gain(ipts,ivm) * litter(ipts,:,ivm,:,:) / total_losses
             soil_bank(:) =  soil_bank(:) + &
                  loss_gain(ipts,ivm) * carbon(ipts,:,ivm) / total_losses
             DO ilev=1,nlevs
                IF(litter_weight_struc(ilev) .GE. min_stomate)THEN
                   struct_ltr_bank(ilev) = struct_ltr_bank(ilev) + &
                        lignin_struc(ipts,ivm,ilev) * &
                        (litter(ipts,istructural,ivm,ilev,icarbon) * ABS(loss_gain(ipts,ivm))) / &
                        litter_weight_struc(ilev)
                ENDIF
                IF(litter_weight_woody(ilev) .GE. min_stomate)THEN
                   woody_ltr_bank(ilev) = woody_ltr_bank(ilev) + &
                        lignin_wood(ipts,ivm,ilev) * &
                        (litter(ipts,iwoody,ivm,ilev,icarbon) * ABS(loss_gain(ipts,ivm))) / &
                        litter_weight_woody(ilev)
                ENDIF
             ENDDO

             fresh_ltr_bank(:,:) = fresh_ltr_bank(:,:) + &
                  loss_gain(ipts,ivm) * fresh_litter(ivm,:,:) / total_losses

             ! In stomate_season.f90
             ! this variable is only calculated if the biomass(ileaf) > 0. As long
             ! as veget_max is 0, this variable does not get a value. We need a value
             ! to calculate ::KF which in turn is needed to calculate the biomass
             ! of the seedlings we will prescribe. At this point in the calculation
             ! the moiavail_growingseason from PFT1 is still zero so it is not 
             ! accounted for yet. Its exact value depends on the the PFT that will
             ! be established on PFT1 and it will be accounted for later in the code
             IF (ivm .GT. 1) THEN

                moi_bank = moi_bank + loss_gain(ipts,ivm) / total_losses * &
                     wstress_season(ipts,ivm)

             ENDIF
             
          ENDIF ! IF (loss_gain(ivm,ipts) < -min_stomate)

       ENDDO ! Loop over # PFT's

       !! 5.2 Land cover gains within a pixel
       !  The consequtive loops over nvm cannot be merged because we first
       !  need to deposit all the available pools on the bank before we
       !  can withdraw the deposits to initialize new PFTs with the weighted
       !  mean of the deposits

       !! 5.2.1 There is no land cover change or there is a loss      
       DO ivm = 1,nvm

          IF ( loss_gain(ipts,ivm) .LE. -min_stomate ) THEN
             
             !  There is no land cover change or there is a loss. In case there 
             !  is no land cover change nothing should be done. The case in which 
             !  there is a loss is already dealt with above - see comment below §5.2 
             !  to read why the case of a loss is not integrated in this IF-statement

             !  Update the orphan flux
             IF (veget_max(ipts,ivm) + loss_gain(ipts,ivm) .GE. min_vegfrac) THEN
                
                ! The PFT was not entirely removed
                orphan_flux_local(ipts,ivm,ivegnew,icarbon) = veget_max(ipts,ivm) + &
                     loss_gain(ipts,ivm)
                WRITE(numout,*) 'Giving an orphan flux here 3: ',ivm,ipts

             ELSE
                
                ! The PFT was entirely removed. This is exactely the case for
                ! which the orphan_flux was introduced. Do not manipulate it.
                orphan_flux_local(ipts,ivm,ivegnew,icarbon) = veget_max(ipts,ivm)
                WRITE(numout,*) 'Giving an orphan flux here 4: ',ivm,ipts

             ENDIF

             !---TEMP---
             IF (ld_lcc .AND. test_grid == ipts) THEN
                WRITE(numout,*) 'check biomass sync for losses ', ivm
                WRITE(numout,*) 'biomass, ', SUM(biomass(ipts,ivm,:,icarbon))
                WRITE(numout,*) 'circ_class_biomass, ',SUM(SUM(circ_class_biomass(ipts,ivm,:,:,icarbon),2)*&
                     circ_class_n(ipts,ivm,:))
             ENDIF
             !---------
 
          ! Increase in the extent of a PFT
          ELSEIF ( loss_gain(ipts,ivm) .GE. min_stomate ) THEN

             !! 5.2.1 Check wether this is the youngest age class
             !  If the redistribution of the LCC map in age classes is correctly 
             !  implemented a new age class should always be established in the 
             !  youngest age class which should be the first of the group. This 
             !  implies that in this case ::ivm is the first of the group and 
             !  ivm + nagec - 1 is the last age class of the group
             IF (is_tree(ivm)) THEN

                igroup = INT(Agec_group(ivm))
                nagec_temp = nagec_pft(igroup)

                IF (agec_group(ivm+nagec_temp-1) .NE. igroup) THEN

                   ! ::ivm and ivm+nagec-1 are in a different ::igroup 
                   ! apparently ::ivm is not the youngest age class
                   WRITE(numout,*) 'Error: problem with the redistribution of the LCC maps'
                   WRITE(numout,*) 'ipts, ivm, nagec_temp:  ',ipts, ivm
                   WRITE(numout,*) 'nagec_temp, igroup:  ',nagec_temp,igroup
                   WRITE(numout,*) 'agec_group(ivm), agec_group(ivm+nagec_temp-1):  ',&
                        agec_group(ivm), agec_group(ivm+nagec_temp-1)
                   CALL ipslerr_p (3,'land_cover_main', 'Cannot find proper age classes! (see output file)','','')
                ENDIF


             ENDIF

             !! 5.2.2 The new PFT does not exist yet
             IF (veget_max(ipts,ivm) .LT. min_stomate) THEN

                !! 5.2.2.1 Initialize new and dummy variables
                new_biomass(:,:,:,:) = zero
                new_circ_class_biomass(:,:,:,:,:) = zero
                new_ind(:,:) = zero
                new_circ_class_n(:,:,:) = zero
                new_lm_lastyearmax(:,:) = zero
                new_age(:,:) = zero
                new_leaf_frac(:,:,:) = zero
                new_co2_to_bm(:,:) = zero
                new_everywhere(:,:) = zero 
                dum_when_growthinit(:,:) = large_value
                dum_npp_longterm(:,:) = zero
                dum_PFTpresent(:,:) = .TRUE.
                dum_senescence(:,:) = .FALSE.
               
                !! 5.2.2.2 Assign value to moiavail_growingseason
                !  Remember to add the moisture from PFT1 that will be present 
                !  if PFT1 is planted with PFT ivm. Note ::loss_gain is here for
                !  PFT1 but ::vir_moiavail_growingseason is for PFT ivm.
!!$                moiavail_growingseason(ipts,ivm) = moi_bank + &
!!$                     vir_moiavail_growingseason(ipts,ivm) * &
!!$                     loss_gain(ipts,1) / total_losses
                ! k_latosa_adapt is adjusted every day. wstress is no longer
                ! used in prescribe
!!$                wstress_season(ipts,ivm) = moi_bank + &
!!$                     vir_wstress_fac(ipts,ivm) * &
!!$                     loss_gain(ipts,1) / total_losses 

                !! 5.2.2.3 Prescribe values for newly established PFTs
                ! Initialize density of individuals, biomass, allocation factors,
                ! leaf age distribution, ... to some reasonable value
                ! ::veget_max is not updated yet, therefore loss_gain is passed in 
                ! the argument list. As ::veget_max is used in the IF-THEN-statements
                ! we won't update it until the end of this routine. If we would
                ! update before that, the same PFT may be subject to conflicting actions
                ! i.e. prescribe a new PFT and then merge it with the other age-classes
                ! in the PFT-group.
                CALL prescribe_prognostic (npts, loss_gain, veget, dt_days, dum_PFTpresent, &
                     new_everywhere, dum_when_growthinit, new_biomass, new_leaf_frac, &
                     new_ind, new_circ_class_n, new_circ_class_biomass, new_co2_to_bm, &
                     forest_managed, KF, &
                     dum_senescence, new_age, dum_npp_longterm, new_lm_lastyearmax, &
                     tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt)

                ! Prescribe the new values
                biomass(ipts,ivm,:,:) = new_biomass(ipts,ivm,:,:)
                ind(ipts,ivm) = new_ind(ipts,ivm)
                circ_class_biomass(ipts,ivm,:,:,:) = new_circ_class_biomass(ipts,ivm,:,:,:)
                circ_class_n(ipts,ivm,:) = new_circ_class_n(ipts,ivm,:)
                lm_lastyearmax(ipts,ivm) = new_lm_lastyearmax(ipts,ivm)
                age(ipts,ivm) = new_age(ipts,ivm)
                leaf_frac(ipts,ivm,:) = new_leaf_frac(ipts,ivm,:)
                co2_to_bm(ipts,ivm) = new_co2_to_bm(ipts,ivm)
                everywhere(ipts,ivm) = new_everywhere(ipts,ivm)
                PFTpresent(ipts,ivm) = .TRUE.

                ! Update the orphan flux
                orphan_flux_local(ipts,ivm,ivegnew,icarbon) = &
                     veget_max(ipts,ivm) + loss_gain(ipts,ivm)
                orphan_flux_local(ipts,ivm,ico2bm,icarbon) = co2_to_bm(ipts,ivm)
                WRITE(numout,*) 'Giving an orphan flux here 2: ',ivm,ipts

                ! Update a couple forestry parameters.  I initialize these to
                ! one instead of zero since forestry is called after this
                ! routine, and it looks at last_cut.  This is not a real
                ! problem since the trees we prescribe generally are the height of
                ! saplings and not seeds.
                age_stand(ipts,ivm)=1
                last_cut(ipts,ivm)=1

                ! Initialize the soil and litter carbon pools. When PFTs were removed
                ! their soil and litter carbon was deposited on the bank. It is assumed
                ! that the newly established PFT are established uniformly on all the
                ! removed PFTs. Hence there is no preferential LCC sequence. For example
                ! we deforest a needle and broadleaved PFT and replace it by a crop and
                ! a grassland the crop and grassland are both established on the former
                ! needle and broadleaved PFT rather than the crop being entirely on the 
                ! former needleleaved PFT and grassland on the former broadleaved PFT.
                ! Further, the LCC-maps should guarantee that the losses equal the gains
                ! which ensures mass preservation. For these reasons we can simply use the
                ! mean soil and litter carbon on the bank (gC m-2) to initialize the soil
                ! and litter crabon of the newly established PFT's. 
                litter(ipts,:,ivm,:,:) = litter_bank(:,:,:)
                carbon(ipts,:,ivm) =  soil_bank(:)
                lignin_struc(ipts,ivm,:) = struct_ltr_bank(:)
                lignin_wood(ipts,ivm,:) = woody_ltr_bank(:)
                bm_to_litter(ipts,ivm,:,:) = fresh_ltr_bank(:,:) 

             !! 5.2.3 The new PFT does already exist 
             ELSEIF (veget_max(ipts,ivm) .GE. min_stomate) THEN
                     
                !  We want to make use of prescribe_prognostic but it should be noted
                !  that this PFT already contains biomass. Therefore we will create a
                !  temporary PFT without biomass which can be used to establish the
                !  new vegetation within this PFT. For most of the INTENT(inout)
                !  variables of prescribe_prognostic we receive temporary variables 
                !  back this helps us to calculate the weighted mean of the newly 
                !  established vegetation and the vegetation that is already there.
                !  For some other variables we receive dummies as we are simply not
                !  going to use these values but will continue using the values of the
                !  vegetation that is already there.

                !! 5.2.3.1 Initialize new and dummy variables
                new_biomass(:,:,:,:) = zero
                new_circ_class_biomass(:,:,:,:,:) = zero
                new_ind(:,:) = zero
                new_circ_class_n(:,:,:) = zero
                new_lm_lastyearmax(:,:) = zero
                new_age(:,:) = zero
                new_leaf_frac(:,:,:) = zero
                new_co2_to_bm(:,:) = zero
                new_everywhere(:,:) = zero 
                dum_when_growthinit(:,:) = large_value
                dum_npp_longterm(:,:) = zero
                dum_PFTpresent(:,:) = .TRUE.
                dum_senescence(:,:) = .FALSE.
                
                !! 5.2.3.2 Merge the new and already available vegetation
                !  Veget_max after PFT expansion. As ::veget_max is used in the 
                !  IF-statements we won't update it until the end of this routine. If 
                !  we would updatebefore that, the same PFT may be subject to 
                !  conflicting actions.
                share_expanded = veget_max(ipts,ivm) / &
                     ( veget_max(ipts,ivm) + loss_gain(ipts,ivm) )

                ! We also need a scaling factor which includes the litter.  This is for growing
                ! a new PFT on the soil mixture left from all the land we cleared.
                DO ilev=1,nlevs
                   DO ilit=1,nlitt
                      IF(litter(ipts,ilit,ivm,ilev,icarbon) .GE. min_stomate)THEN
                         litter_weight_expanded(ilit,ilev) = litter(ipts,ilit,ivm,ilev,icarbon) * &
                              veget_max(ipts,ivm)/ &
                              (litter(ipts,ilit,ivm,ilev,icarbon) * veget_max(ipts,ivm) + &
                              litter_bank(ilit,ilev,icarbon) * loss_gain(ipts,ivm))
                      ELSE
                         litter_weight_expanded(ilit,ilev)=zero
                      ENDIF
                   END DO
                ENDDO

                ! Remember to add the moisture from PFT1 that will be present 
                ! if PFT1 is planted with PFT ivm. Note ::loss_gain is here for
                ! PFT1 but ::vir_wstress_fac is for PFT ivm.
                wstress_season(ipts,ivm) = wstress_season(ipts,ivm) * &
                     share_expanded + (moi_bank + vir_wstress_fac(ipts,ivm) * &
                     loss_gain(ipts,1) / total_losses ) * (un - share_expanded)
                wstress_month(ipts,ivm) = wstress_month(ipts,ivm) * &
                     share_expanded + (moi_bank + vir_wstress_fac(ipts,ivm) * &
                     loss_gain(ipts,1) / total_losses ) * (un - share_expanded)

                !! 5.2.3.3 Initialize the newly established vegetation: 
                !  Initialize the newly established vegetation: density of 
                !  individuals, biomass, allocation factors, leaf age distribution, 
                !  etc to some reasonable value ::veget_max is not updated yet, 
                !  therefore loss_gain is passed in the argument list.
                CALL prescribe_prognostic (npts, loss_gain, veget, dt_days, dum_PFTpresent, &
                     new_everywhere, dum_when_growthinit, new_biomass, new_leaf_frac, &
                     new_ind, new_circ_class_n, new_circ_class_biomass, new_co2_to_bm, &
                     forest_managed, KF, &
                     dum_senescence, new_age, dum_npp_longterm, new_lm_lastyearmax, &
                     tau_eff_leaf, tau_eff_sap, tau_eff_root, k_latosa_adapt)

                !! 5.2.3.3.1 Merge biomass of new and already available trees
                IF (is_tree(ivm)) THEN

                   ! Copy the number of individuals and the biomass of the established 
                   ! vegetation to a temporary variable. In the subroutine ::circ_class_n
                   ! and ::circ_class_biomass will be overwritten with the characteristics
                   ! of the merged vegetation
                   est_circ_class_n(:) = circ_class_n(ipts,ivm,:)
                   est_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,ivm,:,:,:)
                   tmp_circ_class_n(:) = new_circ_class_n(ipts,ivm,:)
                   tmp_circ_class_biomass(:,:,:) = new_circ_class_biomass(ipts,ivm,:,:,:)

                   ! Store circumference (m) in a temporary variable that can be 
                   ! sorted from small to large - note that the dimension of these
                   ! variables are 2*ncirc
                   est_circ_class_dia(:) = wood_to_dia(est_circ_class_biomass(:,:,icarbon),ivm)
                   tmp_circ_class_dia(:) = wood_to_dia(tmp_circ_class_biomass(:,:,icarbon),ivm)

                   ! Merge the biomass        
!!$                   CALL merge_biomass_pfts(npts, share_expanded, moiavail_growingseason, circ_class_n, &
                   CALL merge_biomass_pfts(npts, share_expanded, circ_class_n, &
                        est_circ_class_n, tmp_circ_class_n, circ_class_biomass, est_circ_class_biomass, &
                        tmp_circ_class_biomass, ind, biomass, circ_class_dist, &
                        ipts, ivm, est_circ_class_dia, tmp_circ_class_dia)

                        
                !! 5.2.3.3.2 Merge biomass of new and already available grass or cropland
                ELSE
                                  
                   ! Dilute the existing biomass pools and number of individuals
                   biomass(ipts,ivm,:,:) = share_expanded * biomass(ipts,ivm,:,:) + &
                        (un - share_expanded) * new_biomass(ipts,ivm,:,:)
                   ind(ipts,ivm) = share_expanded * ind(ipts,ivm) + &
                        (un - share_expanded) * new_ind(ipts,ivm)
                   
                   ! Keep ::biomass and circ_class_biomass synchronized
                   IF (ind(ipts,ivm) .GE. min_stomate) THEN

                      circ_class_biomass(ipts,ivm,1,:,:) = biomass(ipts,ivm,:,:)/ind(ipts,ivm)
                      circ_class_n(ipts,ivm,1) = ind(ipts,ivm)

                   ELSE

                      circ_class_biomass(ipts,ivm,1,:,:) = zero
                      circ_class_n(ipts,ivm,1) = zero

                   ENDIF                       

                ENDIF
               
                !! 5.2.3.4 Calculate the PFT characteristics of the merged PFT
                !  Take the weighted mean of the existing vegetation and the new 
                !  vegetation in this PFT. Note that co2_to_bm is in gC. m-2 dt-1 
                !  so we should also take the weighted mean (rather than sum if
                !  this where absolute values).
                lm_lastyearmax(ipts,ivm) = share_expanded * lm_lastyearmax(ipts,ivm) + &
                     (un - share_expanded) * new_lm_lastyearmax(ipts,ivm)
                age(ipts,ivm) = share_expanded * age(ipts,ivm) + &
                     (un - share_expanded) * new_age(ipts,ivm)
                leaf_frac(ipts,ivm,:) = share_expanded * leaf_frac(ipts,ivm,:) + &
                     (un - share_expanded) * new_leaf_frac(ipts,ivm,:)
                co2_to_bm(ipts,ivm) = share_expanded * co2_to_bm(ipts,ivm) + &
                     (un - share_expanded) * new_co2_to_bm(ipts,ivm)

                ! Update the orphan flux
                orphan_flux_local(ipts,ivm,ivegnew,icarbon) = &
                     veget_max(ipts,ivm) + loss_gain(ipts,ivm)
                orphan_flux_local(ipts,ivm,ico2bm,icarbon) = co2_to_bm(ipts,ivm)
                WRITE(numout,*) 'Giving an orphan flux here 1: ',ivm,ipts

                ! Everywhere deals with the migration of vegetation. Copy the
                ! status of the most migrated vegetation for the whole PFT
                everywhere(ipts,ivm) = MAX(everywhere(ipts,ivm), new_everywhere(ipts,ivm))

                ! The new soil&litter pools are the weighted mean of the newly 
                ! established vegetation for that PFT (as deposited on the bank) 
                ! and the vegetation that already exists in that PFT.
                ! NOTE that in case veget_max of PFT1 (bare soil) is increasing
                ! we will also move litter to the bare soil. stomate_litter.f90
                ! was adjusted such that decomposition also happens in PFT
                litter(ipts,:,ivm,:,:) = share_expanded * litter(ipts,:,ivm,:,:) + &
                        (un - share_expanded) * litter_bank(:,:,:)
                carbon(ipts,:,ivm) =  share_expanded * carbon(ipts,:,ivm) + &
                     (un - share_expanded) * soil_bank(:)
                lignin_struc(ipts,ivm,:) = litter_weight_expanded(istructural,:) * lignin_struc(ipts,ivm,:) + &
                     (un - litter_weight_expanded(istructural,:)) * struct_ltr_bank(:)
                lignin_wood(ipts,ivm,:) = litter_weight_expanded(iwoody,:) * lignin_wood(ipts,ivm,:) + &
                     (un - litter_weight_expanded(iwoody,:)) * woody_ltr_bank(:)
                bm_to_litter(ipts,ivm,:,:) = share_expanded * bm_to_litter(ipts,ivm,:,:) + & 
                     (un - share_expanded) * fresh_ltr_bank(:,:)

                ! These next two variables are integers, so we use CEILING to make sure
                ! they stay integers.  FLOOR or INT would also be fine.  The use of
                ! CEILING is arbitrary.
                age_stand(ipts,ivm)  = CEILING(share_expanded * age_stand(ipts,ivm) + &
                     (un - share_expanded) * 1)
                last_cut(ipts,ivm)  = CEILING(share_expanded * last_cut(ipts,ivm) + &
                     (un - share_expanded) * 1)

             ELSE

                WRITE(numout,*) 'Error 1: logic of the IF-statements in land_cover_change'
   
             ENDIF ! New PFT

          ELSEIF(ABS(loss_gain(ipts,ivm)) .LE. 1e-15)THEN
             ! Do nothing.  This is okay, as far as I can tell.
             ! This seems to be the double precision limit.

          ELSE

             WRITE(numout,*) 'Error 2: logic of the IF-statements in land_cover_change'
             WRITE(numout,*) 'ipts, ivm: ',ipts,ivm
             WRITE(numout,*) 'loss_gain: ',loss_gain(ipts,ivm)

          ENDIF ! Land cover change?

       ENDDO  ! Loop over # PFT's

    ENDDO ! Loop over npts
    

 !! 6. Calculate new veget_max values and update variables if PFT was removed 
    
    !  Calculate new veget_max    
    veget_max(:,:) = veget_max(:,:) + loss_gain(:,:)

    !  Update the variables if veget_max is below threshold
    DO ipts = 1,npts
       
       DO ivm = 1,nvm
          IF (veget_max(ipts,ivm) .LT. min_vegfrac) THEN

             veget_max(ipts,ivm) = zero
             lm_lastyearmax(ipts,ivm) = zero
             age(ipts,ivm) = zero
             leaf_frac(ipts,ivm,:) = zero
             co2_to_bm(ipts,ivm) = zero
             everywhere(ipts,ivm) = zero
             litter(ipts,:,ivm,:,:) = zero
             carbon(ipts,:,ivm) = zero
             lignin_struc(ipts,ivm,:) = zero
             lignin_wood(ipts,ivm,:) = zero
             bm_to_litter(ipts,ivm,:,:) = zero
             biomass(ipts,ivm,:,:) = zero
             ind(ipts,ivm) = zero
             PFTpresent(ipts,ivm) = .FALSE.
             ! Keep ::biomass and circ_class_biomass synchronized
             circ_class_biomass(ipts,ivm,:,:,:) = zero
             circ_class_n(ipts,ivm,:) = zero

             age_stand(ipts,ivm)=1
             last_cut(ipts,ivm)=1

          ENDIF

       ENDDO

    ENDDO

    IF (ld_lcc) THEN
       WRITE(numout,*) 'Final values before mass balance'
       CALL print_lcc_error_message(npts,test_grid,veget_max_new_adj,veget_max,veget_max_new_group,&
            veget_max_group,delta_veget,loss_gain,4)
    ENDIF

!! 6. Mass balance closure
    
    !! 6.1 Calculate components of the mass balance
    pool_end(:,:,:) = zero
    DO iele = 1,nelements

       IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Final pool 2 ',pool_start(test_grid,test_pft,iele)
          ENDIF
       ENDIF

       ! Biomass pool + bm_to_litter
       DO ipar = 1,nparts
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) + bm_to_litter(:,:,ipar,iele)) * &
               veget_max(:,:)
          IF(test_grid > 0)THEN
             IF(test_pft < nvm)THEN
                WRITE(numout,'(A,I5,10ES20.10)') 'Final biomass ',ipar,biomass(test_grid,test_pft,ipar,iele) , &
                     bm_to_litter(test_grid,test_pft,ipar,iele), &
                     veget_max(test_grid,test_pft)
             ENDIF
          ENDIF
       ENDDO

       IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Final pool 3 ',pool_end(test_grid,test_pft,iele)
          ENDIF
       ENDIF

       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilit = 1,nlitt
          DO ilev = 1,nlevs
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  litter(:,ilit,:,ilev,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Final pool 4 ',pool_end(test_grid,test_pft,iele)
          ENDIF
       ENDIF

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO      

       IF(test_grid > 0)THEN
          IF(test_pft < nvm)THEN
             WRITE(numout,'(A,10ES20.10)') 'Final pool 5 ',pool_end(test_grid,test_pft,iele)
          ENDIF
       ENDIF

    ENDDO

    ! The biomass harvest pool is expressed in gC pixel-1 So, it 
    ! shouldn't be multiplied by veget_max but it should be divided
    ! by pixel_area to obtain gC m-2. The difference between 
    ! pool_start and pool_end was accumulated over dt, therefore, 
    ! the units of ipoolchange are gC m-2 dt-1.
    DO ivm = 1,nvm
       pool_end(:,ivm,icarbon) = pool_end(:,ivm,icarbon) + &
            SUM(harvest_pool(:,ivm,:,icarbon),2) / &
            pixel_area(:)
    ENDDO

    !! 6.2 Calculate mass balance
    check_intern(:,:,iatm2land,icarbon) = orphan_flux_local(:,:,ico2bm,icarbon) * orphan_flux_local(:,:,ivegnew,icarbon)
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO
    mb_lcc_error(:)=SUM(closure_intern(:,:,icarbon),2)

    !! 6.3 Write outcome of the check
    !  Sum over ivm because of LCC
    DO ipts=1,npts
       IF(ABS(SUM(closure_intern(ipts,:,icarbon))) .LE. min_stomate)THEN
          IF (ld_massbal) WRITE(numout,*) 'Mass balance closure in land_cover_main'
       ELSE
          WRITE(numout,*) 'Error: mass balance is not closed in land_cover_main',&
               ABS(SUM(closure_intern(ipts,:,icarbon)))
          DO ivm=1,nvm
             WRITE(numout,*) '   ipts,ivm; ', ipts,ivm
             WRITE(numout,*) '   Difference is, ', closure_intern(ipts,ivm,icarbon)
             WRITE(numout,*) '   pool_end,pool_start: ', pool_end(ipts,ivm,icarbon), &
                  pool_start(ipts,ivm,icarbon)
             WRITE(numout,*) '   orphan_fluxes: ', orphan_flux_local(ipts,ivm,ico2bm,icarbon),&
                  orphan_flux_local(ipts,ivm,ivegnew,icarbon)
             WRITE(numout,*) '   veget_max: ', veget_max(ipts,ivm)
             
          ENDDO
          IF(ld_stop)THEN
             CALL ipslerr_p (3,'land_cover_main', 'Mass balance error.','','')
          ENDIF
       ENDIF
    ENDDO

    ! Here we recombine the orphan fluxes calculated in this subroutine with
    ! the values which exist outside this subroutine.
    orphan_flux(:,:,:,:)=orphan_flux(:,:,:,:)+orphan_flux_local(:,:,:,:)


    !+++++++++++++++++++++++++ TEMP ++++++++++++++++++
    !+++++++ TEMP +++++
    ! Check to see if the nobio fraction has changed
    DO ipts=1,npts
       frac_nobio_end(ipts,1)=un-SUM(veget_max(ipts,:))
       IF(abs(frac_nobio_start(ipts,1) - frac_nobio_end(ipts,1)) .GT. min_stomate)THEN
          WRITE(numout,*) 'ERROR: nobio fraction changed during LCC!'
          WRITE(numout,*) 'ipts, frac_nobio_start(ipts,1): ',ipts, frac_nobio_start(ipts,1)
          WRITE(numout,*) 'ipts, frac_nobio_end(ipts,1): ',ipts, frac_nobio_end(ipts,1)
          IF(.NOT. lignore_lcc_stops) STOP
       ENDIF
    ENDDO
    !+++++++++++++++++++

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving land_cover_change_main'

  END SUBROUTINE land_cover_change_main


! ================================================================================================================================
!! SUBROUTINE   : age_class_distr
!!
!>\BRIEF        Redistribute biomass, litter, soilcarbon and water across
!!              the age classes
!!
!! DESCRIPTION  : Following growth, the trees from an age class may have become
!! too big to belong to this age class. The biomass, litter, soilcarbon and
!! soil water then need to be moved from one age class to the next age class.
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE age_class_distr(npts, circ_class_n, biomass, circ_class_biomass, &
       veget_max, ind, circ_class_dist, wstress_season, &
       lm_lastyearmax, age, leaf_frac, co2_to_bm, &
       everywhere, litter, carbon, lignin_struc, &
       lignin_wood, bm_to_litter, PFTpresent, when_growthinit,&
       forest_managed, KF, senescence, &
       npp_longterm, gpp_daily, leaf_age, lab_fac, &
       gdd_from_growthinit, gdd_midwinter, time_hum_min, gdd_m5_dormance, &
       ncd_dormance, moiavail_month, moiavail_week, ngd_minus5, &
       gpp_week, resp_maint, resp_growth, npp_daily, &
       use_reserve, t_photo_stress, rue_longterm, mai, pai, &
       mai_count, previous_wood_volume, moiavail_growingseason, &
       MatrixA, MatrixV, VectorB, VectorU, age_stand, last_cut, &
       k_latosa_adapt)

    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    INTEGER, INTENT(in)                                :: npts                !! Domain size - number of pixels (unitless)
    REAL(r_std), DIMENSION(:), INTENT(in)              :: circ_class_dist     !! The probability distribution of trees in a circ
 
                                                                              !! class in case of a redistribution (unitless). 
    !! 0.2 Output variables


    !! 0.3 Modified variables

    LOGICAL, DIMENSION(:,:), INTENT(inout)             :: PFTpresent          !! Tab indicating which PFTs are present in 
                                                                              !! each pixel
    LOGICAL, DIMENSION(:,:), INTENT(inout)             :: senescence          !! Flag for setting senescence stage (only 
                                                                              !! for deciduous trees)
    INTEGER(i_std), DIMENSION (:,:), INTENT(inout)     :: forest_managed      !! forest management flag (is the forest 
                                                                              !! being managed?)
     REAL(r_std), DIMENSION(:,:), INTENT(inout)        :: moiavail_month      !! "Monthly" moisture availability (0 to 1, 
                                                                              !! unitless) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: moiavail_week       !! "Weekly" moisture availability 
                                                                              !! (0 to 1, unitless)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gpp_week            !! Mean weekly gross primary productivity 
                                                                              !! @tex $(gC m^{-2} day^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: ngd_minus5          !! Number of growing days (days), threshold 
                                                                              !! -5 deg C (for phenology)   
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: resp_maint          !! Maintenance respiration  
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: resp_growth         !! Growth respiration  
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: npp_daily           !! Net primary productivity 
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: use_reserve         !! Mass taken from carbohydrate reserve 
                                                                              !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: t_photo_stress      !! Temperature stress for CEXCHANGE 
                                                                              !! photosynthesis (0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: rue_longterm        !! Longterm radiation use efficiency 
                                                                              !! (??units??) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: when_growthinit     !! How many days ago was the beginning of 
                                                                              !! the growing season (days)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: lab_fac             !! Activity of labile pool factor (??units??)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: KF                  !! Scaling factor to convert sapwood mass
                                                                              !! into leaf mass (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: k_latosa_adapt      !! Leaf to sapwood area adapted for long 
                                                                              !! term water stress (m)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: npp_longterm        !! "Long term" mean yearly primary productivity 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: ind                 !! Number of individuals at the stand level
                                                                              !! @tex $(m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: veget_max           !! "maximal" coverage fraction of a PFT on the ground
                                                                              !! May sum to
                                                                              !! less than unity if the pixel has
                                                                              !! nobio area. (unitless, 0-1)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: wstress_season      !! Water stress factor, based on hum_rel_daily
                                                                              !! (unitless, 0-1)   
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: lm_lastyearmax      !! last year's maximum leaf mass for each PFT 
                                                                              !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: everywhere          !! is the PFT everywhere in the grid box or 
                                                                              !! very localized (after its introduction) (?)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: age                 !! mean age (years)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: co2_to_bm           !! CO2 taken from the atmosphere to get C to create  
                                                                              !! the seedlings @tex (gC.m^{-2}dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gpp_daily           !! Daily gross primary productivity  
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: time_hum_min        !! Time elapsed since strongest moisture 
                                                                              !! availability (days) 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gdd_midwinter       !! Growing degree days (K), since midwinter 
                                                                              !! (for phenology) - this is written to the
                                                                              !!  history files 
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gdd_from_growthinit !! growing degree days, since growthinit 
                                                                              !! for crops
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: gdd_m5_dormance     !! Growing degree days (K), threshold -5 deg 
                                                                              !! C (for phenology)
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: ncd_dormance        !! Number of chilling days (days), since 
                                                                              !! leaves were lost (for phenology) 
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: circ_class_n        !! Number of individuals in each circ class
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: lignin_struc        !! ratio Lignine/Carbon in structural litter,
                                                                              !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: lignin_wood         !! ratio Lignine/Carbon in woody litter,
                                                                              !! above and below ground
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: carbon              !! carbon pool: active, slow, or passive 
                                                                              !! @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: leaf_frac           !! fraction of leaves in leaf age class (unitless;0-1)
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)       :: leaf_age            !! Leaf age (days)
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: bm_to_litter        !! Transfer of biomass to litter 
                                                                              !! @tex $(gC m^{-2} dtslow^{-1})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)     :: biomass             !! Stand level biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout)   :: circ_class_biomass  !! Biomass components of the model tree within a 
                                                                              !! circumference class
                                                                              !! class @tex $(g C ind^{-1})$ @endtex  
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout)   :: litter              !! metabolic and structural litter, above and 
                                                                              !! below ground @tex ($gC m^{-2}$) @endtex
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)    :: mai                 !! The mean annual increment
                                                                              !! @tex $(m**3 / m**2 / year)$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)    :: pai                 !! The period annual increment
                                                                              !! @tex $(m**3 / m**2 / year)$ @endtex          
    INTEGER(i_std), DIMENSION(npts,nvm),INTENT(inout)  :: mai_count           !! The number of times we've
                                                                              !! calculated the volume increment
                                                                              !! for a stand 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(inout)    :: previous_wood_volume!! The volume of the tree trunks
                                                                              !! in a stand for the previous year.
                                                                              !! @tex $(m**3 / m**2 )$ @endtex
    REAL(r_std), DIMENSION(:,:), INTENT(inout)         :: moiavail_growingseason  !! Mean growingseason moisture 
                                                                              !! availability (0 to 1, unitless)
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), &
                                         INTENT(inout) :: MatrixA             !! Matrix containing the fluxes between the
                                                                              !! carbon pools per sechiba time step 
                                                                              !! @tex $(gC.m^2.day^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools), &
                                         INTENT(inout) :: VectorB             !! Vector containing the litter increase per
                                                                              !! sechiba time step
                                                                              !! @tex $(gC m^{-2})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nbpools,nbpools), &
                                         INTENT(inout) :: MatrixV             !!
    REAL(r_std), DIMENSION(npts,nvm,nbpools), &
                                         INTENT(inout) :: VectorU             !!
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(inout) :: age_stand           !! Age of the forest stand (years)  
    INTEGER(i_std), DIMENSION(npts,nvm), INTENT(inout) :: last_cut            !! Years since last thinning (years)

    !! 0.4 Local variables

    ! +++CHECK+++
    ! Externalize
     REAL(r_std), DIMENSION(nagec)                     :: age_class_bound     !! Boundaries of the age classes (m)
    ! +++++++++++

    INTEGER(i_std)                                     :: ipts,ivm,igroup     !! Indeces(unitless)
    INTEGER(i_std)                                     :: iele,ipar,ipft      !! Indeces(unitless)
    INTEGER(i_std)                                     :: iagec,imbc,icirc    !! Indeces(unitless)
    INTEGER(i_std)                                     :: ilit,ilev,icarb     !! Indeces(unitless)
    INTEGER(i_std)                                     :: ivma                !! Indeces(unitless)
    REAL(r_std)                                        :: share_expanded      !! Share of the veget_max of the existing vegetation
                                                                              !! within a PFT over the total veget_max following 
                                                                              !! expansion of that PFT (unitless, 0-1)
    REAL(r_std), DIMENSION(ncirc)                      :: circ_class_ba       !! basal area of the model tree in each
                                                                              !! circ class @tex $(m^{2} m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(npts,nvm)                   :: qm_dia              !! Quadratic mean diameter of the stand (m)
    REAL(r_std), DIMENSION(ncirc)                      :: est_circ_class_n    !! Temporary variable for circ_class_n of the 
                                                                              !! established vegetation 
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                      :: tmp_circ_class_n    !! Temporary variable for circ_class_n of the 
                                                                              !! established vegetation 
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)     :: est_circ_class_biomass !! Temporary variable for circ_class_biomass 
                                                                              !! of the established vegetation 
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc,nparts,nelements)     :: tmp_circ_class_biomass !! Temporary variable for circ_class_biomass 
                                                                              !! of the established vegetation
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(ncirc)                      :: est_circ_class_dia  !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    REAL(r_std), DIMENSION(ncirc)                      :: tmp_circ_class_dia  !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    REAL(r_std), DIMENSION(npts,nvm,nmbcomp,nelements) :: check_intern        !! Contains the components of the internal
                                                                              !! mass balance chech for this routine
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: closure_intern      !! Check closure of internal mass balance
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: pool_start          !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(npts,nvm,nelements)         :: pool_end            !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(nelements)                  :: temp_start          !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(nelements)                  :: temp_end            !! Start and end pool of this routine 
                                                                              !! @tex $(gC pixel^{-1} dt^{-1})$ @endtex
    REAL(r_std), DIMENSION(nlitt,nlevs)                :: litter_weight_expanded !! The fraction of litter on the expanded
                                                                              !! PFT.
                                                                              !! @tex $-$ @endtex
!_ ================================================================================================================================
  
    IF (bavard.GE.3) WRITE(numout,*) 'Entering age class distribution'


 !! 1. Initialize

    !! 1.1 Initialize variables
    !+++CHECK++++
    !Externalize
    IF(nagec == 4)THEN
       age_class_bound(1) = 0.07
       age_class_bound(2) = 0.20
       age_class_bound(3) = 0.40
       age_class_bound(4) = 5.
    ELSEIF(nagec == 1)THEN
       age_class_bound(1) = 5.0
    ELSE
       WRITE(numout,*) 'You need to externalize age_class_bound, or come up with a case '
       WRITE(numout,*) 'for this number of age classes.'
       WRITE(numout,*) 'nagec = ',nagec
       STOP
    ENDIF
    !++++++++++++

    !! 1.2 Initialize check for mass balance closure
    !  The mass balance is calculated at the end of this routine
    !  in section 3. Initial biomass and harvest pool all other
    !  relevant pools were just set to zero.
    pool_start(:,:,:) = zero
    DO iele = 1,nelements
       
       ! co2_to_bm
       pool_start(:,:,iele) = pool_start(:,:,iele) + co2_to_bm(:,:)

       ! Biomass pool + bm_to_litter
       DO ipar = 1,nparts
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               (biomass(:,:,ipar,iele) + bm_to_litter(:,:,ipar,iele)) * &
               veget_max(:,:)
       ENDDO

       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilit = 1,nlitt
          DO ilev = 1,nlevs
             pool_start(:,:,iele) = pool_start(:,:,iele) + &
                  litter(:,ilit,:,ilev,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_start(:,:,iele) = pool_start(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO

    ENDDO


 !! 2. Redistribute the biomass across the age classes

    !  Following growth, the trees from an age class may have become too big to 
    !  belong to this age class. The biomass, litter, soilcarbon and soil
    !  water then need to be moved from one age class to the next age class.
    DO ipts = 1,npts

       ! The agec_group lets us know which PFTs belong in which age class groups.
       !  I assume that the structure is linear, that is, if PFTs 16, 17, and 18 all
       ! below to group 10, PFT 16 is the youngest, PFT 17 is the second-younest, and
       ! PFT 18 is the oldest (largest diameter).  I also assume that all PFTs in 
       ! an age class are grouped together in the input file.  In the example above,
       ! this means that PFT 19 cannot be in group 10 unless PFT 18 is also in group 10.
       

       ! This loops over all the PFTs that we have ignoring age classes
       DO ivma=1,nvmap

          ivm=start_index(ivma)

          ! If we only have a single age class for this
          ! PFT, we can skip it.
          IF(nagec_pft(ivma) .EQ. 1)CYCLE

          ! Only forest PFTs have age classes
          IF (is_tree(ivm)) THEN

             !! 2.1 Check if the trees still belong to this age class
             !  Note that the term age class is used but that the classes used in the
             !  code are not defined on an age criterion. Instead a diameter criterion
             !  is used. The reason to do so is that growth depends on the rule of
             !  Deleuze and Dhote that in turn is based on basal area. More stable
             !  model behaiour is therefore expected when the diameter range within an
             !  age class is controled. Also different PFTs grow at a different speeds.
             !  If age would have been used to define the classes, the diameter range
             !  within the first age class would have been much larger in the 
             !  tropics compared to the boreal zone. When Deleuze and Dhote is then
             !  applied the model could start to behave differently in the tropics and
             !  boreal zone. These differences in behaviour are thought to be better 
             !  controled by implementing a diameter criterion.
             !  As a reminder age classes (::nagec) control the diameter/height
             !  variation across stands, ::ncirc controls the diameter/height variation
             !  within a stand 
             DO iagec = nagec_pft(ivma),1,-1

                ipft = ivm+iagec-1

                !! 2.1.1 Basal area at the tree level (m2 tree-1)
                circ_class_ba(:) = &
                     wood_to_ba(circ_class_biomass(ipts,ipft,:,:,icarbon),ipft)

                ! Convert basal area in quadratic mean stand diameter (m)
                IF (ind(ipts,ipft) .NE. zero) THEN

                   qm_dia(ipts,ipft) = SQRT( 4/pi * SUM(circ_class_ba(:) * &
                        circ_class_n(ipts,ipft,:)) / ind(ipts,ipft) )

                ELSE

                   qm_dia(ipts,ipft) = zero

                ENDIF

                !! 2.2.2 Check whether quadratic mean diameter exceeds boundaries 
                !  Check whether quadratic mean diameter exceeds boundaries of
                !  the age class.
                IF(ld_agec)THEN
                   WRITE(numout,*) 'Checking to merge for: '
                   WRITE(numout,*) 'ipft,ivm,iagec,ipts: ',ipft,ivm,iagec,ipts
                   WRITE(numout,*) 'nagec_pft,qm_dia,age_class_bound: ',nagec_pft(ivma),&
                        qm_dia(ipts,ipft),age_class_bound(iagec)
                ENDIF

                IF ( (iagec .EQ. nagec_pft(ivma)) .AND. &
                     qm_dia(ipts,ipft) .GT. age_class_bound(iagec) ) THEN
                 
                   ! If these conditions are satisfied our tree diameter is
                   ! very unrealist
                   WRITE(numout,*) 'WARNING: tree diameter exceeds (m): ', &
                        age_class_bound(iagec) 
  
                ELSEIF ( (iagec .NE. nagec_pft(ivma)) .AND. &
                     qm_dia(ipts,ipft) .GT. age_class_bound(iagec)) THEN

                   IF(ld_agec)THEN
                      WRITE(numout,*) 'Merging biomass'
                      WRITE(numout,*) 'ipts,ipft,iagec: ',ipts,ipft,iagec
                      WRITE(numout,*) 'age_class_bound(:): ',age_class_bound(:)
                      WRITE(numout,*) 'qm_dia: ',qm_dia(ipts,ipft)

                   ENDIF

                   !! 2.2.3 Merge biomass
                   !  Biomass of two age classes needs to be merged. The established
                   !  vegetation is stored in ipft+1, the new vegetation is stored in
                   !  ipft
                   share_expanded = veget_max(ipts,ipft+1) / &
                        ( veget_max(ipts,ipft+1) + veget_max(ipts,ipft) )
                   ! We also need a scaling factor which includes the litter
                   DO ilev=1,nlevs
                      DO ilit=1,nlitt
                         IF(litter(ipts,ilit,ipft,ilev,icarbon) .GE. min_stomate)THEN
                            litter_weight_expanded(ilit,ilev)=litter(ipts,ilit,ipft+1,ilev,icarbon) * veget_max(ipts,ipft+1)/ &
                                 (litter(ipts,ilit,ipft+1,ilev,icarbon) * veget_max(ipts,ipft+1) + &
                                 litter(ipts,ilit,ipft,ilev,icarbon) * veget_max(ipts,ipft))
                         ELSE
                            litter_weight_expanded(ilit,ilev)=zero
                         ENDIF
                      END DO
                   ENDDO
                      
                   ! Assign value to moiavail_growingseason, this value needs to be 
                   ! copied because it is only calculated when there is leaf mass present 
                   moiavail_growingseason(ipts,ipft+1) = moiavail_growingseason(ipts,ipft+1) * &
                        share_expanded + moiavail_growingseason(ipts,ipft) * (un - share_expanded)
                   ! I'm not sure if wstress_season needs to be here since it is recalculated from
                   ! moiavail_growingseason every day.
                   wstress_season(ipts,ipft+1) = wstress_season(ipts,ipft+1) * &
                          share_expanded + wstress_season(ipts,ipft) * (un - share_expanded)

                   ! Copy the number of individuals and the biomass of the established 
                   ! vegetation to a temporary variable. In the subroutine ::circ_class_n
                   ! and ::circ_class_biomass will be overwritten with the characteristics
                   ! of the merged vegetation
                   est_circ_class_n(:) = circ_class_n(ipts,ipft+1,:)
                   est_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,ipft+1,:,:,:)                 
                   tmp_circ_class_n(:) = circ_class_n(ipts,ipft,:)
                   tmp_circ_class_biomass(:,:,:) = circ_class_biomass(ipts,ipft,:,:,:)
 
                   ! Store circumference (m) in a temporary variable that can be 
                   ! sorted from small to large - note that the dimension of these
                   ! variables are 2*ncirc
                   est_circ_class_dia(:) = wood_to_dia(est_circ_class_biomass(:,:,icarbon),ipft+1)
                   tmp_circ_class_dia(:) = wood_to_dia(tmp_circ_class_biomass(:,:,icarbon),ivm)

                   ! Initiliaze mass balance of the merging procedure
                   ! MB was calculated outside the subroutine merge_biomass_pfts
                   ! because veget_max needs to be updated. Veget_max cannot be 
                   ! updated in the subroutine because the way it is called from 
                   ! the subroutine land_cover_change (where veget_max can only be
                   ! updated at the end of the whole subroutine. Therefore, this
                   ! check was included here.
                   temp_start(:) = zero
                   DO ipar = 1,nparts
                      DO iele = 1,nelements
                         DO icirc = 1,ncirc
                            temp_start(iele) = temp_start(iele) + &
                                 SUM(circ_class_biomass(ipts,:,icirc,ipar,iele) * &
                                 circ_class_n(ipts,:,icirc) * veget_max(ipts,:))
                         ENDDO
                      ENDDO
                   ENDDO

                   ! Merge the biomass of the two age classes
                   CALL merge_biomass_pfts(npts, share_expanded, circ_class_n, &
                        est_circ_class_n, tmp_circ_class_n, circ_class_biomass, est_circ_class_biomass, &
                        tmp_circ_class_biomass, ind, biomass, circ_class_dist, ipts, &
                        ipft+1, est_circ_class_dia, tmp_circ_class_dia)

                   !! 2.2.4 Empty the age class that was merged and update veget_max
                   circ_class_n(ipts,ipft,:) = zero
                   ind(ipts,ipft) = zero
                   circ_class_biomass(ipts,ipft,:,:,:) = zero
                   biomass(ipts,ipft,:,:) = zero
                   veget_max(ipts,ipft+1) = veget_max(ipts,ipft+1) + veget_max(ipts,ipft)
                   veget_max(ipts,ipft) = zero

                   ! Mass balance of the merging procedure
                   temp_end(:) = zero
                   DO ipar = 1,nparts
                      DO iele = 1,nelements
                         DO icirc = 1,ncirc
                            temp_end(iele) = temp_end(iele) + &
                                 SUM(circ_class_biomass(ipts,:,icirc,ipar,iele) * &
                                 circ_class_n(ipts,:,icirc) * veget_max(ipts,:))
                         ENDDO
                      ENDDO
                   ENDDO
 
                   IF (temp_end(icarbon) - temp_start(icarbon) .LT. min_stomate .AND. &
                        temp_end(icarbon) - temp_start(icarbon) .GT. -min_stomate) THEN
                      IF (ld_massbal) WRITE(numout,*) 'Mass balance closure merge_biomass_pfts'
                   ELSE
                      WRITE(numout,*) 'Error: mass balance is not closed in merge_biomass_pfts'
                      WRITE(numout,*) '   Difference is, ',temp_end(icarbon) - temp_start(icarbon) 
                   ENDIF

                   !! 2.3 Calculate the PFT characteristics of the merged PFT
                   !  Take the weighted mean of the existing vegetation and the new 
                   !  vegetation in this PFT. Note that co2_to_bm is in gC. m-2 dt-1 
                   !  so we should also take the weighted mean (rather than sum if
                   !  this where absolute values).
                   lm_lastyearmax(ipts,ipft+1) = share_expanded * lm_lastyearmax(ipts,ipft+1) + &
                        (un - share_expanded) * lm_lastyearmax(ipts,ipft)
                   lm_lastyearmax(ipts,ipft) = zero
                   age(ipts,ipft+1) = share_expanded * age(ipts,ipft+1) + &
                        (un - share_expanded) * age(ipts,ipft)
                   age(ipts,ipft) = zero
                   leaf_frac(ipts,ipft+1,:) = share_expanded * leaf_frac(ipts,ipft+1,:) + &
                        (un - share_expanded) * leaf_frac(ipts,ipft,:)
                   leaf_frac(ipts,ipft,:) = zero
                   leaf_age(ipts,ipft+1,:) = share_expanded * leaf_age(ipts,ipft+1,:) + &
                        (un - share_expanded) * leaf_age(ipts,ipft,:)
                   leaf_age(ipts,ipft,:) = zero
                   lab_fac(ipts,ipft+1) = share_expanded * lab_fac(ipts,ipft+1) + &
                        (un - share_expanded) * lab_fac(ipts,ipft)
                   lab_fac(ipts,ipft) = zero
                   co2_to_bm(ipts,ipft+1) = share_expanded * co2_to_bm(ipts,ipft+1) + &
                        (un - share_expanded) * co2_to_bm(ipts,ipft)
                   co2_to_bm(ipts,ipft) = zero

                   ! Everywhere deals with the migration of vegetation. Copy the
                   ! status of the most migrated vegetation for the whole PFT
                   everywhere(ipts,ipft+1) = MAX(everywhere(ipts,ipft), everywhere(ipts,ipft+1))
                   everywhere(ipts,ipft) = zero

                   ! The new soil&litter pools are the weighted mean of the newly 
                   ! established vegetation for that PFT and the soil&litter pools
                   ! of the vegetation that already exists in that PFT.
                   ! Since it is not only the amount of vegetation present (veget_max) but also
                   ! the amount of structural litter (litter) that is important, we have to 
                   ! weight by both items here.
                   DO ilev=1,nlevs
                      lignin_struc(ipts,ipft+1,ilev) = litter_weight_expanded(istructural,ilev) * lignin_struc(ipts,ipft+1,ilev) + &
                           (un - litter_weight_expanded(istructural,ilev)) * lignin_struc(ipts,ipft,ilev) 
                      lignin_struc(ipts,ipft,ilev) = zero
                      lignin_wood(ipts,ipft+1,ilev) = &
                           litter_weight_expanded(iwoody,ilev) * lignin_wood(ipts,ipft+1,ilev) + &
                           (un - litter_weight_expanded(iwoody,ilev) ) * lignin_wood(ipts,ipft,ilev) 
                      lignin_wood(ipts,ipft,ilev) = zero 
                   ENDDO
                   litter(ipts,:,ipft+1,:,:) = share_expanded * litter(ipts,:,ipft+1,:,:) + &
                        (un - share_expanded) * litter(ipts,:,ipft,:,:)
                   litter(ipts,:,ipft,:,:) = zero
                   carbon(ipts,:,ipft+1) =  share_expanded * carbon(ipts,:,ipft+1) + &
                        (un - share_expanded) * carbon(ipts,:,ipft)
                   carbon(ipts,:,ipft) = zero 
                   bm_to_litter(ipts,ipft+1,:,:) = share_expanded * bm_to_litter(ipts,ipft+1,:,:) + & 
                        (un - share_expanded) * bm_to_litter(ipts,ipft,:,:)
                   bm_to_litter(ipts,ipft,:,:) = zero

                   ! Copy variables that depend on ::when_growthinit
                   when_growthinit(ipts,ipft+1) = share_expanded * when_growthinit(ipts,ipft+1) + &
                        (un - share_expanded) * when_growthinit(ipts,ipft)
                   when_growthinit(ipts,ipft) = zero
                   gdd_from_growthinit(ipts,ipft+1) = share_expanded * &
                        gdd_from_growthinit(ipts,ipft+1) + &
                        (un - share_expanded) * gdd_from_growthinit(ipts,ipft)
                   gdd_from_growthinit(ipts,ipft) = zero
                   gdd_midwinter(ipts,ipft+1) = share_expanded * gdd_midwinter(ipts,ipft+1) + &
                        (un - share_expanded) * gdd_midwinter(ipts,ipft)
                   gdd_midwinter(ipts,ipft) = zero
                   time_hum_min(ipts,ipft+1) = share_expanded * time_hum_min(ipts,ipft+1) + &
                        (un - share_expanded) * time_hum_min(ipts,ipft)
                   time_hum_min(ipts,ipft) = zero
                   gdd_m5_dormance(ipts,ipft+1) = share_expanded * gdd_m5_dormance(ipts,ipft+1) + &
                        (un - share_expanded) * gdd_m5_dormance(ipts,ipft)
                   gdd_m5_dormance(ipts,ipft) = zero
                   ncd_dormance(ipts,ipft+1) = share_expanded * ncd_dormance(ipts,ipft+1) + &
                        (un - share_expanded) * ncd_dormance(ipts,ipft)
                   ncd_dormance(ipts,ipft) = zero
                   moiavail_month(ipts,ipft+1) = share_expanded * moiavail_month(ipts,ipft+1) + &
                        (un - share_expanded) * moiavail_month(ipts,ipft)
                   moiavail_month(ipts,ipft) = zero
                   moiavail_week(ipts,ipft+1) = share_expanded * moiavail_week(ipts,ipft+1) + &
                        (un - share_expanded) * moiavail_week(ipts,ipft)
                   moiavail_week(ipts,ipft) = zero
                   ngd_minus5(ipts,ipft+1) = share_expanded * ngd_minus5(ipts,ipft+1) + &
                        (un - share_expanded) * ngd_minus5(ipts,ipft)
                   ngd_minus5(ipts,ipft) = zero
            
                   ! Copy remaining properties 
                   PFTpresent(ipts,ipft+1) = PFTpresent(ipts,ipft)
                   PFTpresent(ipts,ipft) = .FALSE.
                   forest_managed(ipts,ipft+1) = forest_managed(ipts,ipft)
                   forest_managed(ipts,ipft) = 1
                   KF(ipts,ipft+1) = share_expanded * KF(ipts,ipft+1) + &
                        (un - share_expanded) * KF(ipts,ipft)
                   KF(ipts,ipft) = zero
                   k_latosa_adapt(ipts,ipft+1) = share_expanded * k_latosa_adapt(ipts,ipft+1) + &
                        (un - share_expanded) * k_latosa_adapt(ipts,ipft)
                   k_latosa_adapt(ipts,ipft) = zero
                   senescence(ipts,ipft+1) = senescence(ipts,ipft)
                   senescence(ipts,ipft) = .FALSE.
                   npp_longterm(ipts,ipft+1) = share_expanded * npp_longterm(ipts,ipft+1) + &
                        (un - share_expanded) * npp_longterm(ipts,ipft)
                   npp_longterm(ipts,ipft) = zero
                   gpp_daily(ipts,ipft+1) = share_expanded * gpp_daily(ipts,ipft+1) + &
                        (un - share_expanded) * gpp_daily(ipts,ipft)
                   gpp_daily(ipts,ipft) = zero 
                   gpp_week(ipts,ipft+1) = share_expanded * gpp_week(ipts,ipft+1) + &
                        (un - share_expanded) * gpp_week(ipts,ipft)
                   gpp_week(ipts,ipft) = zero
                   resp_maint(ipts,ipft+1) = share_expanded * resp_maint(ipts,ipft+1) + &
                        (un - share_expanded) * resp_maint(ipts,ipft) 
                   resp_maint(ipts,ipft) = zero
                   resp_growth(ipts,ipft+1) = share_expanded * resp_growth(ipts,ipft+1) + &
                        (un - share_expanded) * resp_growth(ipts,ipft) 
                   resp_growth(ipts,ipft) = zero
                   npp_daily(ipts,ipft+1) = share_expanded * npp_daily(ipts,ipft+1) + &
                        (un - share_expanded) * npp_daily(ipts,ipft) 
                   npp_daily(ipts,ipft) = zero
                   use_reserve(ipts,ipft+1) = share_expanded * use_reserve(ipts,ipft+1) + &
                        (un - share_expanded) * use_reserve(ipts,ipft) 
                   use_reserve(ipts,ipft) = zero
                   t_photo_stress(ipts,ipft+1) = share_expanded * t_photo_stress(ipts,ipft+1) + &
                        (un - share_expanded) * t_photo_stress(ipts,ipft) 
                   t_photo_stress(ipts,ipft) = zero
!!$                   lai_target(ipts,ipft+1) = share_expanded * lai_target(ipts,ipft+1) + &
!!$                        (un - share_expanded) * lai_target(ipts,ipft) 
!!$                   lai_target(ipts,ipft) = zero
                   rue_longterm(ipts,ipft+1)  = share_expanded * rue_longterm(ipts,ipft+1) + &
                        (un - share_expanded) * rue_longterm(ipts,ipft)
                   rue_longterm(ipts,ipft) = zero

                   ! Some things for forestry
                   mai(ipts,ipft+1)  = share_expanded * mai(ipts,ipft+1) + &
                        (un - share_expanded) * mai(ipts,ipft)
                   mai(ipts,ipft) = zero
                   pai(ipts,ipft+1)  = share_expanded * pai(ipts,ipft+1) + &
                        (un - share_expanded) * pai(ipts,ipft)
                   pai(ipts,ipft) = zero
                   ! We need an integer for mai_count.  The use of CEILING here is
                   ! arbitrary.  Maybe FLOOR would work better in some cases.
                   mai_count(ipts,ipft+1)  = CEILING(share_expanded * mai_count(ipts,ipft+1) + &
                        (un - share_expanded) * mai_count(ipts,ipft))
                   mai_count(ipts,ipft) = 0
                   previous_wood_volume(ipts,ipft+1)  = share_expanded * previous_wood_volume(ipts,ipft+1) + &
                        (un - share_expanded) * previous_wood_volume(ipts,ipft)
                   previous_wood_volume(ipts,ipft) = zero
                   ! These next two variables are integers, so we use CEILING to make sure
                   ! they stay integers.  FLOOR or INT would also be fine.  The use of
                   ! CEILING is arbitrary.
                   age_stand(ipts,ipft+1)  = CEILING(share_expanded * age_stand(ipts,ipft+1) + &
                        (un - share_expanded) * age_stand(ipts,ipft))
                   age_stand(ipts,ipft) = zero
                   last_cut(ipts,ipft+1)  = CEILING(share_expanded * last_cut(ipts,ipft+1) + &
                        (un - share_expanded) * last_cut(ipts,ipft))
                   last_cut(ipts,ipft) = zero

                   ! If we are doing an analytical spinup, we need to merge
                   ! the matrices.  During our spinup, only one age class in a given
                   ! PFT will be populated.  Therefore, we just need to move
                   ! the matrix from one PFT to the next.  If there is a chance that
                   ! there is already biomass in the new PFT and things need to be merged,
                   ! you should rethink this!
                   IF (spinup_analytic) THEN
                      IF(share_expanded .LT. min_stomate)THEN
                         MatrixV(ipts,ipft+1,:,:)=MatrixV(ipts,ipft,:,:)
                         MatrixA(ipts,ipft+1,:,:)=MatrixA(ipts,ipft,:,:)
                         VectorU(ipts,ipft+1,:)=VectorU(ipts,ipft,:)
                         VectorB(ipts,ipft+1,:)=VectorB(ipts,ipft,:)
                         MatrixV(ipts,ipft,:,:)=zero
                         MatrixA(ipts,ipft,:,:)=zero
                         VectorU(ipts,ipft,:)=zero
                         VectorB(ipts,ipft,:)=zero
                      ELSE
                         WRITE(numout,*) 'Merge problem with spinup.'
                         WRITE(numout,*) 'ipts,ipft: ',ipts,ipft
                         WRITE(numout,*) 'share_expanded: ',share_expanded
                         CALL ipslerr_p (3,'age_class_distr', 'Trying to merge spinup matrices into a PFT with',&
                              'existing biomass.  Stopping because we are not sure what to do.','')
                      ENDIF
                   ENDIF

                ENDIF

             ENDDO ! loop over nagec_pft(ivma)

          ! Grasses and croplands
          ELSE

             ! Don't have to do anything
             
          ENDIF ! is_tree(ivm)

       ENDDO ! Looping over real PFTs

    ENDDO ! loop over #pixels - domain size


!! 3. Mass balance closure
    
    !! 3.1 Calculate components of the mass balance
    pool_end(:,:,:) = zero
    DO iele = 1,nelements

       ! co2_to_bm
       pool_end(:,:,iele) = pool_end(:,:,iele) + co2_to_bm(:,:)

       ! Biomass pool + bm_to_litter
       DO ipar = 1,nparts
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               (biomass(:,:,ipar,iele) + bm_to_litter(:,:,ipar,iele)) * &
               veget_max(:,:)
       ENDDO

       ! Litter pool (gC m-2) *  (m2 m-2) 
       DO ilit = 1,nlitt
          DO ilev = 1,nlevs
             pool_end(:,:,iele) = pool_end(:,:,iele) + &
                  litter(:,ilit,:,ilev,iele) * veget_max(:,:)
          ENDDO
       ENDDO

       ! Soil carbon (gC m-2) *  (m2 m-2)
       DO icarb = 1,ncarb
          pool_end(:,:,iele) = pool_end(:,:,iele) + &
               carbon(:,icarb,:) * veget_max(:,:)
       ENDDO
    ENDDO

    !! 3.2 Calculate mass balance
    check_intern(:,:,iatm2land,icarbon) = zero 
    check_intern(:,:,iland2atm,icarbon) = -un * zero
    check_intern(:,:,ilat2out,icarbon) = zero
    check_intern(:,:,ilat2in,icarbon) = -un * zero
    check_intern(:,:,ipoolchange,icarbon) = -un * (pool_end(:,:,icarbon) - pool_start(:,:,icarbon))
    closure_intern = zero
    DO imbc = 1,nmbcomp
       closure_intern(:,:,icarbon) = closure_intern(:,:,icarbon) + check_intern(:,:,imbc,icarbon)
    ENDDO

    !! 3.3 Write outcome of the check
    !  Sum over ivm because of age class redistribution
    DO ipts = 1,npts
       IF (SUM(closure_intern(ipts,:,icarbon)) .LT. min_stomate .AND. &
            SUM(closure_intern(ipts,:,icarbon)) .GT. -min_stomate) THEN
          IF (ld_massbal) WRITE(numout,*) 'Mass balance closure: age_class_distr', ipts
       ELSE
          WRITE(numout,*) 'Error: mass balance is not closed in age_class_distr'
          WRITE(numout,*) '   Difference, ipts, ', ipts, SUM(closure_intern(ipts,:,icarbon)) 
       ENDIF
    ENDDO

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving age class distribution'
    
  END SUBROUTINE age_class_distr



! ================================================================================================================================
!! SUBROUTINE   : merge_biomass_pfts
!!
!>\BRIEF        Redistribute biomass, litter, soilcarbon and water across
!!              the age classes
!!
!! DESCRIPTION  : Following growth, the trees from an age class may have too
!! big to still belong to this age class. The biomass, litter, soilcarbon and
!! soil water then need to be moved from one age class to the next age class.
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  SUBROUTINE merge_biomass_pfts(npts, share_expanded, circ_class_n, est_circ_class_n, &
       tmp_circ_class_n, circ_class_biomass, est_circ_class_biomass, &
       tmp_circ_class_biomass, ind, biomass, circ_class_dist, ipts, &
       ivm, est_circ_class_dia, tmp_circ_class_dia)

    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables
    
    INTEGER, INTENT(in)                              :: npts                  !! Domain size - number of pixels (unitless)
    REAL(r_std), INTENT(in)                          :: share_expanded        !! Share of the veget_max of the existing vegetation
                                                                              !! within a PFT over the total veget_max following 
                                                                              !! expansion of that PFT (unitless, 0-1)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: circ_class_dist       !! The probability distribution of trees in a circ
                                                                              !! class in case of a redistribution (unitless). 
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: est_circ_class_biomass!! Temporary variable for circ_class_biomass 
                                                                              !! of the established vegetation
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(in)        :: tmp_circ_class_biomass!! Temporary variable for circ_class_biomass 
                                                                              !! of the new vegetation 
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)            :: est_circ_class_n      !! Temporary variable for circ_class_n
                                                                              !! of the established vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tmp_circ_class_n      !! Temporary variable for circ_class_n
                                                                              !! of the new vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:), INTENT(in)            :: est_circ_class_dia    !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    REAL(r_std), DIMENSION(:), INTENT(in)            :: tmp_circ_class_dia    !! Variable to store temporary values for 
                                                                              !! circ_class (m)
    INTEGER(i_std), INTENT(IN)                       :: ipts, ivm             !! Indices for point and PFT (unitless)

    !! 0.2 Output variables


    !! 0.3 Modified variables
    REAL(r_std), DIMENSION(:,:), INTENT(inout)       :: ind                   !! Number of individuals at the stand level
                                                                              !! @tex $(m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(:,:,:,:), INTENT(inout)   :: biomass               !! Stand level biomass @tex $(gC.m^{-2})$ @endtex
    REAL(r_std), DIMENSION(:,:,:,:,:), INTENT(inout) :: circ_class_biomass    !! circ_class_biomass of the merged vegetation 
                                                                              !! @tex $(g C ind^{-1})$ @endtex
    REAL(r_std), DIMENSION(:,:,:), INTENT(inout)     :: circ_class_n          !! circ_class_n of the merged vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex

    !! 0.4 Local variables
    INTEGER(i_std)                                   :: ipart                 !! Indices (unitless)
    INTEGER(i_std)                                   :: icir, jcir, iele      !! Indices (unitless)
    REAL(r_std)                                      :: trees_needed          !! Number of trees leftthat should be present in a 
                                                                              !! circumference class when merging the established
                                                                              !! and newly established vegetation
                                                                              !! @tex $(ind m^{-2})$ @endtex 
    REAL(r_std)                                      :: tempa, tempb          !! Variable to store a temporary value (unit varies)
    REAL(r_std), DIMENSION(nparts,nelements)         :: tempc                 !! Variable to store a temporary value of
                                                                              !! circ_class_biomass  
    REAL(r_std), DIMENSION(ncirc)                    :: exp_circ_class_n      !! Temporary variable for circ_class_n of the
                                                                              !! expanded PFT @tex $(ind m^{-2})$ @endtex 
    REAL(r_std), DIMENSION(2*ncirc)                  :: trees_left            !! Number of trees left in a circumference class when
                                                                              !! merging the available and newly established 
                                                                              !! vegetation @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(2*ncirc)                  :: temp_circ_class       !! Temporary variable for circ_class of the expanded
                                                                              !! PFT for sorting the availble and newly established
                                                                              !! circumference classes (m)
    REAL(r_std), DIMENSION(2*ncirc)                  :: temp_circ_class_n     !! Temporary variable for circ_class_n of the expanded
                                                                              !! PFT for sorting the availble and newly established
                                                                              !! circumference classes @tex $(ind m^{-2})$ @endtex
    REAL(r_std), DIMENSION(2*ncirc,nparts,nelements) :: temp_circ_class_biomass !! Temporary variable for sorting circ_class_biomass
                                                                              !! @tex $(g C ind^{-1})$ @endtex

!_ ================================================================================================================================

 IF (bavard.GE.3) WRITE(numout,*) 'Entering merging biomass of pfts'

 !! 1. Initialize

    !! 1.1 Initialize variables
   

 !! 2. Merge biomass, litter, soilcarbon and soil water

    !  We have already ::ncirc classes in this PFT and we add another 
    !  ::ncirc through calling prescribe_prognostic. What we want to 
    !  do is to merge these 2*ncirc into ncirc circumference classes.     
    !  We need to populate the new distribution of trees among
    !  the circumference classes, and move the heartwood and sap masses 
    !  to the new distributions. We also need to move the other pools.  
    !  This is more tricky because the allometric relations for the 
    !  new tree sizes will NOT give the same amount of biomass for
    !  the non-woody pools. Let us first try it just distributing
    !  everything equally, and then if that doesn't work we can
    !  redistribute the non-woody pools more cleverly, even if that 
    !  means we change the total amount of biomass we have in this
    !  stand.  I want to try it this way because we conserve biomass 
    !  this way, and I hope that the stress on the trees will be
    !  small enough that it doesn't cause problems. Redistribution
    !  is not expected to de-stabilize the allocation too much is the
    !  diameter range within an age-class is not too big (or in other
    !  words if we define a sufficiently large number of age classes).
 
    !! 2.1 Calculate new distribution
    !  The first step is to decide of the distribution of individuals 
    !  that we want in each class, for example, a uniform or exponential 
    !  distribution (::circ_class_dist). The total number of individuals
    !  that needs to be distributed is the weighted sum of the number of 
    !  individuals (#/m-2) that is already available and the the 
    !  individuals that are newly established.  
    IF (share_expanded .LT. min_stomate) THEN

       ! The new PFT is still empty so simply copy the current tree 
       ! distribution into the new PFT
       exp_circ_class_n(:) = tmp_circ_class_n(:)

    ELSE

       ! The new PFT contains trees, merge the tree distributions. The
       ! new tree distribution follows a prescribed distribution
       exp_circ_class_n(:) = circ_class_dist(:) * &
            SUM( share_expanded * est_circ_class_n(:) + &
            (un-share_expanded) * tmp_circ_class_n(:) )

    ENDIF   


    !! 2.2 Copy both distributions into a single variable
    DO icir = 1,2*ncirc

       IF (icir .LE. ncirc) THEN

          ! Circumference classes, number of individuals and biomass
          ! of the available vegetation (m). Weight by the area share
          ! such that we don't need to do this further down. Don't
          ! weight the circumference and biomass as those variables are
          ! expressed per tree. This helps to simplify the equations.
          temp_circ_class(icir) = pi * est_circ_class_dia(icir)
          temp_circ_class_n(icir) = share_expanded * &
               est_circ_class_n(icir)
          temp_circ_class_biomass(icir,:,:) = &
               est_circ_class_biomass(icir,:,:)  

       ELSEIF ( (icir .GT. ncirc) .AND. (icir .LE. 2*ncirc) ) THEN

          ! Circumference classes, number of individuals and biomass
          ! of the newly established vegetation (m). Weight by the area 
          ! share such that we don't need to do this further down. Don't
          ! weight the circumference and biomass as those variables are
          ! expressed per tree. This helps to simplify the equations.
          temp_circ_class(icir) = pi * tmp_circ_class_dia(icir-ncirc)
          temp_circ_class_n(icir) = (un - share_expanded) * &
               tmp_circ_class_n(icir-ncirc)
          temp_circ_class_biomass(icir,:,:) = &
               tmp_circ_class_biomass(icir-ncirc,:,:)

       ELSE

          WRITE(numout,*) 'Error - problem with the dimensions of a do-loop '

       ENDIF

    ENDDO !loop over 2*ncirc


    !! 2.3 Sort the biomass and number of trees according to their 
    !  circumference 
    DO jcir = 2,2*ncirc

       tempa = temp_circ_class(jcir)
       tempb = temp_circ_class_n(jcir)
       tempc(:,:) = temp_circ_class_biomass(jcir,:,:)

       DO icir = jcir-1,1,-1

          IF (temp_circ_class(icir) .LE. tempa) EXIT
          temp_circ_class(icir+1) = temp_circ_class(icir)
          temp_circ_class_n(icir+1) = temp_circ_class_n(icir)
          temp_circ_class_biomass(icir+1,:,:) = &
               temp_circ_class_biomass(icir,:,:)

       ENDDO

       temp_circ_class(icir+1) = tempa
       temp_circ_class_n(icir+1) = tempb
       temp_circ_class_biomass(icir+1,:,:) = tempc(:,:)

    ENDDO
   

    !! 2.4 Merge into a single distribution with ncirc classes
    !  Merge the 2*ncirc classes in ncirc classes. Empty one circ class
    !  after the other going from the smallest to the biggest. Here we
    !  still have 2*ncirc dimensions
    circ_class_biomass(ipts,ivm,:,:,:) = zero
    trees_left(:) = temp_circ_class_n(:)
    DO icir=1,ncirc

       ! This is the number of trees in the ncirc circ classes thus
       ! AFTER the merge happened 
       trees_needed = exp_circ_class_n(icir)
       DO jcir = 1,2*ncirc

          IF (trees_needed .LT. min_stomate) EXIT

          IF (trees_left(jcir) .GE. trees_needed) THEN

             ! We can get all the trees we need from this class
             ! and don't have to continue searching. Note that
             ! the units of circ_class_biomass have temporarly 
             ! changed to gC m-2
             circ_class_biomass(ipts,ivm,icir,:,:) = &
                  circ_class_biomass(ipts,ivm,icir,:,:) + &
                  temp_circ_class_biomass(jcir,:,:) * &
                  trees_needed
             trees_left(jcir) = trees_left(jcir) - trees_needed
             trees_needed = zero

             EXIT

          ELSE

             ! The number of trees in this class are not sufficient, so
             ! we need to move all of them to the new class. Note that
             ! the units of circ_class_biomass have temporarly changed 
             ! to gC m-2
             circ_class_biomass(ipts,ivm,icir,:,:) = &
                  circ_class_biomass(ipts,ivm,icir,:,:) + &
                  temp_circ_class_biomass(jcir,:,:) * &
                  trees_left(jcir)
             trees_needed = trees_needed - trees_left(jcir)
             trees_left(jcir) = zero

          ENDIF

       ENDDO ! jcir=1,ncirc   

    ENDDO ! icir=1,ncirc


    !! 2.5 convert units back to the original units (gC tree-1) 
    !  Right now, circ_class_biomass gives the total biomass
    !  in each class, but it should only be for a model tree. 
    !  So let's normalize it. And, update the variables that 
    !  pass this information around
    DO icir=1,ncirc

       circ_class_biomass(ipts,ivm,icir,:,:) = &
            circ_class_biomass(ipts,ivm,icir,:,:) / exp_circ_class_n(icir)
       circ_class_n(ipts,ivm,icir) = exp_circ_class_n(icir)         

    ENDDO
 
    ! Synchronize circ_class_biomass and biomass and 
    ! circ_class_n and ind
    ind(ipts,ivm) = SUM(circ_class_n(ipts,ivm,:))
    DO iele = 1,nelements

       DO ipart = 1,nparts

          biomass(ipts,ivm,ipart,iele) = &
               SUM(circ_class_biomass(ipts,ivm,:,ipart,iele) * &
               circ_class_n(ipts,ivm,:))

       ENDDO

    ENDDO

    IF (bavard.GE.4) WRITE(numout,*) 'Leaving merging biomass of pfts'
    
  END SUBROUTINE merge_biomass_pfts

! ================================================================================================================================
!! SUBROUTINE   : print_lcc_error_message
!!
!>\BRIEF        
!!
!! DESCRIPTION  : 
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE print_lcc_error_message(npts,ipts,veget_max_new,veget_max,&
       veget_max_new_group,veget_max_group,delta_veget,loss_gain,flag)

    IMPLICIT NONE
    
  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    INTEGER, INTENT(in)                                :: npts                 !! Domain size - number of pixels (unitless)
    INTEGER, INTENT(in)                                :: ipts                 !! Location of current gridsquare
    INTEGER, INTENT(in)                                :: flag                 !! Indicates what to print
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)       :: veget_max            !! "maximal" coverage fraction of a PFT (LAI ->
                                                                               !! infinity) on ground.  (unitless) 
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)       :: veget_max_new        !! new "maximal" coverage fraction of a PFT (LAI
                                                                               !! -> infinity) on ground. 
    REAL(r_std),DIMENSION(npts,nvmap), INTENT(in)      :: delta_veget          !! changes in "maximal" coverage fraction of PFT 
    REAL(r_std), DIMENSION(npts,nvmap) , INTENT(in)    :: veget_max_new_group  !! Total veget_max_new of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvmap) , INTENT(in)    :: veget_max_group      !! Total veget_max of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm)                   :: loss_gain            !! The same as delta_veget but distributed of all
                                                                               !! age classes and thus taking the age-classes into 
                                                                               !! account (unitless, 0-1)

    !! 0.2 Output variables


    !! 0.3 Modified variables

    !! 0.4 Local variables
    INTEGER(i_std)                                    :: ivm,ivma  !! Indices
!_ ================================================================================================================================

    IF(ipts .LT. 1)THEN
       WRITE(numout,*) 'ERROR: Passing a bad point to print_lcc_error_message.'
       WRITE(numout,*) 'ipts, test_grid: ',ipts,test_grid
       RETURN
    ENDIF

    SELECT CASE(flag)
    CASE(1)

       ! Case 1
       WRITE(numout,*) 'ipts ivma veget_max_group veget_max_new_group'
       DO ivma=1,nvmap
          WRITE(numout,'(2I7,2F14.10)') ipts,ivma,veget_max_group(ipts,ivma),&
               veget_max_new_group(ipts,ivma)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_max_group(ipts,:)),&
            SUM(veget_max_new_group(ipts,:))

    CASE(2)

       ! Case 2
       WRITE(numout,*) 'ivm veget_max veget_max_new'
       DO ivm=1,nvm
          WRITE(numout,'(2I7,2F14.10)') ipts,ivm,veget_max(ipts,ivm),&
               veget_max_new(ipts,ivm)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_max(ipts,:)),&
            SUM(veget_max_new(ipts,:))

    CASE(3)

       ! Case 3
       WRITE(numout,*) 'ivm veget_max_group delta_veget'
       DO ivma=1,nvmap
          WRITE(numout,'(2I7,3F14.10)') ipts,ivma,veget_max_group(ipts,ivma),&
               delta_veget(ipts,ivma)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_max_group(ipts,:)),&
            SUM(delta_veget(ipts,:))
    CASE(4)

       !Case 4
       WRITE(numout,*) 'ipts ivm veget_max loss_gain'
       DO ivm=1,nvm
         WRITE(numout,'(2I7,3F14.10)') ipts,ivm,&
               veget_max(ipts,ivm),loss_gain(ipts,ivm)
       ENDDO
       WRITE(numout,*) 'SUMS: ',SUM(veget_max(ipts,:)),&
            SUM(loss_gain(ipts,:))

   CASE(5)

       ! Case 5
       WRITE(numout,*) 'ivm agec_group'
       DO ivm=1,nvm
          WRITE(numout,'(2I8)') ivm,agec_group(ivm)
       ENDDO

    CASE default

       ! Default case
       WRITE(numout,*) 'ERROR: Not sure what you want me to print in LCC.'
    END SELECT
    
     
  END SUBROUTINE print_lcc_error_message


! ================================================================================================================================
!! SUBROUTINE   : adjust_delta_veget
!!
!>\BRIEF        
!!
!! DESCRIPTION  : 
!!
!! RECENT CHANGE(S) : 
!!
!! MAIN OUTPUT VARIABLE(S) :  
!!
!! REFERENCES   : None
!!
!! FLOWCHART    : 
!! \n
!_ ================================================================================================================================

  
  SUBROUTINE adjust_delta_veget(npts,veget_max,veget_max_group,veget_max_new,&
       veget_max_new_group,loss_gain,delta_veget)

  !! 0. Variable and parameter declaration 
    
    !! 0.1 Input variables

    INTEGER, INTENT(in)                                :: npts                 !! Domain size - number of pixels (unitless)
    REAL(r_std), DIMENSION(npts,nvmap),INTENT(in)               :: veget_max_group        !! Total veget_max of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)           :: veget_max_new    !! new "maximal" coverage fraction of a PFT (LAI
                                                                                  !! -> infinity) on ground. 
    REAL(r_std), DIMENSION(npts,nvmap), INTENT(in)               :: veget_max_new_group    !! Total veget_max_new of all age clases within a PFT group 
                                                                               !! (unitless, 0-1)
    REAL(r_std), DIMENSION(npts,nvm), INTENT(in)           :: veget_max        !! "maximal" coverage fraction of a PFT (LAI ->
                                                                                  !! infinity) on ground.  (unitless) 

    !! 0.2 Output variables
    REAL(r_std), DIMENSION(npts,nvm), INTENT(out)                 :: loss_gain              !! The same as delta_veget but distributed of all
                                                                                           !! age classes and thus taking the age-classes into 


    !! 0.3 Modified variables
    REAL(r_std),DIMENSION(npts,nvmap), INTENT(inout)      :: delta_veget          !! changes in "maximal" coverage fraction of PFT 

    !! 0.4 Local variables
    REAL(r_std), DIMENSION(npts,nvmap)                    :: delta_veget_old        !! A temporary copy of delta_veget.
    INTEGER(i_std)                                    :: ivm,ivma,ipts  !! Indices
    INTEGER(i_std)                                    :: jvma,igroup,iyoung  !! Indices
    REAL(r_std)                                      :: excess_veg_add         !! 
    REAL(r_std)                                      :: excess_veg_rem         !! 
    REAL(r_std)                                      :: veg_add_tot            !! 
    REAL(r_std)                                      :: difference             !! How much leftover veget_max we have
    INTEGER(i_std)                                   :: ncount                 !! The number of PFTs with significant changes
                                                                               !! in vegetation.
    INTEGER(i_std)                                   :: nagec_local            !! The number of age classes in this map PFT.

!_ ================================================================================================================================


    loss_gain(:,:)=zero

    ! Here we have to be careful.  There is a variable, min_vegfrac, which checks some things
    ! below to make sure we do not end up with land that is below a certain size.  This seems to be
    ! mainly for the DGVM, but it affects us here, too.  If our final veget_max is too low,
    ! this will cause problems.  Therefore, adjust them beforehand.  

    delta_veget_old(:,:)=delta_veget(:,:)
    DO ipts=1,npts

       
       IF(ld_lcc)THEN
          IF(test_grid == ipts)THEN
             WRITE(numout,*) 'Readjusting LCC fluxes for grid: ',ipts
          ENDIF
       ENDIF

!! 4. Distribute delta_veget over all age-classes
    
    !  ::delta_veget has been calculated for the sum of the age classes
    !  within a PFT. ::delta_veget thus needs to be distributed over 
    !  all the age classes. This distribution requires a rule which 
    !  in the absence of better information is based on the assumption
    !  that a land cover change does not care about the age classes.
    !  Following a forestry logic, one would change an old forest into 
    !  a grassland because that would result in an income from the wood
    !  harvest. However, here we assume that LCC follows a spatialized 
    !  economic need and therefore does not respect the forestry logic.
    !  In other words, when a farmer needs more land, he will not cut
    !  spatialy distributed old stands but will target a block of land
    !  bordering his current agricultural lands. By doing so forest
    !  of all age classes will be cut when LCC is the underlying 
    !  motivation. The reasoning applies for deforestation driven by 
    !  urbanisation. This assumption translates in that the land cover
    !  fraction of all age classes will decrease proportionally to their
    !  share when the PFT experiences a loss. When the PFT experiences an
    !  increase in the PFT cover this only happens in the youngest age *
    !  class.

    ! There is one more constraint.  The new vegetation fraction at
       ! the end of the calculation must be larger than min_vegfrac.  It might
       ! not be if we remove the wrong amount of vegetation.  If veget_max
       ! is greater than zero but less than min_vegfrac after accounting
       ! for loss_gain, we kill the whole PFT and move the leftover to
       ! bare soil.  This leftover should be a small number.
 
       ! This is a clean up case.  We should not have absolute values of loss_gain
       ! or delta_veget which are less than min_stomate.  This carbon will get
       ! lost otherwise.  First let's do delta_veget.  We will have to do something
       ! at the end to be sure nothing has gone wrong with loss_gain, too.
       excess_veg_add=zero
       excess_veg_rem=zero
       DO ivma=1,nvmap
          IF ((delta_veget(ipts,ivma) .LT. min_stomate) .AND. &
               (delta_veget(ipts,ivma) .GT. zero)) THEN

             IF(ld_lcc .AND. ipts == test_grid)THEN
                WRITE(numout,*) 'delta_veget between zero and min_stomate, adding ',&
                     ivma,delta_veget(ipts,ivma)
             ENDIF

             ! We want to add just a tiny amount of land.  Add it somewhere else.
             excess_veg_add=excess_veg_add+delta_veget(ipts,ivma)
             delta_veget(ipts,ivma)=zero

          ELSEIF((delta_veget(ipts,ivma) .LT. zero) .AND. &
               (delta_veget(ipts,ivma) .GT. -min_stomate))THEN
             
             IF(ld_lcc .AND. ipts == test_grid)THEN
                WRITE(numout,*) 'delta_veget between zero and -min_stomate, removing ',&
                     ivma,delta_veget(ipts,ivma)
             ENDIF

             ! We want to remove just a tiny amount of land.  Remove it somewhere else.
             excess_veg_rem=excess_veg_rem+delta_veget(ipts,ivma)
             delta_veget(ipts,ivma)=zero

          ENDIF

       ENDDO


       ! From here on, we are only concerned with loss_gain.  We will ignore
       ! delta_veget after creating loss_gain, and then we will reconstruct
       ! delta_veget at the end from the changes we have made to loss_gain.
       DO ivm=1,nvm

          igroup=agec_group(ivm)
          iyoung=start_index(igroup)

          ! Only forest PFTs have age classes
          IF (is_tree(ivm)) THEN

             ! PFT cover area decreases (loss)
             IF (delta_veget(ipts,igroup) .LE. -min_stomate) THEN

                ! Calculate the proportinal cover change 

                loss_gain(ipts,ivm) = veget_max(ipts,ivm) / &
                        veget_max_group(ipts,igroup) * delta_veget(ipts,igroup)

             ! PFT cover area increases (gain)
             ELSE

                ! The first age-class in a PFT group is the youngest
                ! and that is the age-class were the change in cover fraction
                ! takes place
                IF(ivm == iyoung)THEN
                   loss_gain(ipts,ivm) = delta_veget(ipts,igroup)
                ELSE

                   ! The remaining age-classes in that PFT don't experience a
                   ! change in cover fraction  

                   loss_gain(ipts,ivm) = zero

                ENDIF ! looking for youngest age class

             ENDIF

          ! Grasses and croplands
          ELSE

             ! There are no age-classes for grasses and crops, hence, 
             ! delta_veget should not be distributed over the age classes
             loss_gain(ipts,ivm) = delta_veget(ipts,igroup)

          ENDIF ! is_tree(ivm)

       ENDDO ! looping over PFTs

       IF (ld_lcc) THEN
          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'First estimate of loss_gain '
             
             CALL print_lcc_error_message(npts,test_grid,veget_max_new,veget_max,veget_max_new_group,&
                  veget_max_group,delta_veget,loss_gain,4)
          ENDIF
       ENDIF

       ! It is possible that we try to add a very small amount to a PFT
       ! which does not exist.  We cannot have a veget_max between
       ! min_stomate and min_vegfrac.  We will remove such small
       ! changes here, but we need to be sure to add them somewhere else.
       DO ivm=1,nvm

          IF(veget_max(ipts,ivm) .LT. min_stomate)THEN


             IF((loss_gain(ipts,ivm) .GT. zero) .AND. &
                  (loss_gain(ipts,ivm) .LE. min_vegfrac))THEN
                IF(ld_lcc .AND. ipts == test_grid)THEN
                   WRITE(numout,*) 'Trying to add too small of loss_gain to a PFT ' // &
                        'which does not exist.  Correcting.'
                   WRITE(numout,*) 'ipts, ivm, min_vegfrac: ',ipts, ivm, min_vegfrac
                   WRITE(numout,*) 'veget_max,loss_gain (before): ',&
                        veget_max(ipts,ivm),loss_gain(ipts,ivm)
                ENDIF
                excess_veg_add=excess_veg_add+loss_gain(ipts,ivm)
                loss_gain(ipts,ivm)=zero
                
             ENDIF
             
          ENDIF

       ENDDO ! looping over all PFTs

       ! This is a first readjustment due to the fact that creating
       ! loss_gain proportational to veget_max might generate a
       ! new veget_max for a given PFT which is below min_fracveg.
       DO ivma=1,nvmap

          iyoung=start_index(ivma)
          nagec_local=nagec_pft(ivma)

          ! Only a concern if we have age classes
          IF (nagec_local .GT. 1) THEN

             ! Will any veget_maxes created by these loss_gains
             ! be below the minmum vegetation fraction?

             DO ivm=iyoung,iyoung+nagec_local-1
                IF(loss_gain(ipts,ivm) .LT. zero)THEN
                   ! We also need to check if it's greater than zero, since we might want
                   ! to remove the whole PFT, which is perfectly okay.
                   IF(veget_max(ipts,ivm)-ABS(loss_gain(ipts,ivm)) .LE. min_vegfrac .AND. &
                        (veget_max(ipts,ivm)-ABS(loss_gain(ipts,ivm)) .NE. zero))THEN
                      IF(ld_lcc .AND. ipts == test_grid)THEN
                         WRITE(numout,*) 'Results in a veget_max below min_vegfrac!  Correcting.'
                         WRITE(numout,*) 'ipts, ivm, min_vegfrac: ',ipts, ivm, min_vegfrac
                         WRITE(numout,*) 'veget_max,loss_gain (before): ',&
                              veget_max(ipts,ivm),loss_gain(ipts,ivm)
                      ENDIF
                      ! We kill the whole PFT.  This means we have to add a gain somewhere
                      ! to compensate, probably bare soil.
                      excess_veg_add=excess_veg_add+(veget_max(ipts,ivm)-ABS(loss_gain(ipts,ivm)))
                      loss_gain(ipts,ivm)=-veget_max(ipts,ivm)
                   ENDIF
                ENDIF
             ENDDO

          ENDIF ! checking if we have age classes 

       ENDDO ! looping over the map PFTs

       ! Now we do one final check to make sure that our loss_gain isn't too small.
       ! It should be fairly rare that we actually do something here.  We have
       ! already done the same check above for delta_veget, so if we don't have
       ! age classes there will be nothing to do here.  This is required because
       ! some of the actions in LCC are not done if loss_gain is below min_stomate,
       ! so we would have a mass balance error otherwise.
       DO ivma=1,nvmap

          iyoung=start_index(ivma)
          nagec_local=nagec_pft(ivma)

          ! Only a concern if we have age classes
          IF (nagec_local .GT. 1) THEN

             ! Will any veget_maxes created by these loss_gains
             ! be below the minmum vegetation fraction?

             DO ivm=iyoung,iyoung+nagec_local-1
                IF ((loss_gain(ipts,ivm) .LT. min_stomate) .AND. &
                     (loss_gain(ipts,ivm) .GT. zero)) THEN
                   
                   IF(ld_lcc .AND. ipts == test_grid)THEN
                      WRITE(numout,*) 'loss_gain between zero and min_stomate, adding ',&
                           ivm,ivma,delta_veget(ipts,ivma),loss_gain(ipts,ivm)
                   ENDIF

                  ! We want to add just a tiny amount of land.  Add it somewhere else.
                   excess_veg_add=excess_veg_add+loss_gain(ipts,ivm)
                   loss_gain(ipts,ivm)=zero
                   
                ELSEIF((loss_gain(ipts,ivm) .LT. zero) .AND. &
                     (loss_gain(ipts,ivm) .GT. -min_stomate))THEN
                   
                   IF(ld_lcc .AND. ipts == test_grid)THEN
                      WRITE(numout,*) 'loss_gain between zero and -min_stomate, removing ',&
                           ivm,ivma,delta_veget(ipts,ivma),loss_gain(ipts,ivm)
                   ENDIF

                   ! We want to add just a tiny amount of land.  Add it somewhere else.
                   excess_veg_rem=excess_veg_rem+loss_gain(ipts,ivm)
                   loss_gain(ipts,ivm)=zero
                   
                ENDIF

             ENDDO ! loop over the age classes for this PFT

          ENDIF ! checking if we have age classes 

       ENDDO ! looping over the map PFTs


       ! If we have extra land to add/remove, let's change the loss_gain of bare soil.
       ! This should, in general, be a small number.
       IF (ld_lcc) THEN
          IF(ipts == test_grid)THEN
             WRITE(numout,*) 'Extra vegetation to add: '
             WRITE(numout,*) 'excess_veg_add: ',excess_veg_add
             WRITE(numout,*) 'Extra vegetation to remove: '
             WRITE(numout,*) 'excess_veg_rem: ',excess_veg_rem

             WRITE(numout,*) 'Readjusting loss_gain to make sure an age class does not fall too low.'

             CALL print_lcc_error_message(npts,ipts,veget_max_new,veget_max,veget_max_new_group,&
                  veget_max_group,delta_veget,loss_gain,4)

          ENDIF

       ENDIF

       ! Our loss_gain should not lead to a bare soil fraction below min_vegfrac.
       ! Start with PFT 1 (bare soil) and try to put all the excess into a single PFT.
       ! If we cannot do that, throw an error. It's possible that we can flip the sign
       ! of the loss_gain by doing this, but that should not cause any
       ! serious problem.  We just won't be 100% inline with the maps, but as long
       ! as min_vegfrac exists, we won't be anyways.

       ! We loop over the map PFTs here.  We don't want to loop over all the PFTs
       ! and check, because we might end up adding area to an age class which
       ! is not the youngest that way, which would be bad.
       excess:DO ivma=1,nvmap
          ! Do we have vegetation?
          ivm=start_index(ivma)
          IF(veget_max(ipts,ivm) .GE. min_stomate)THEN

             ! Can we add the difference without falling below min_vegfrac?
             IF((veget_max(ipts,ivm)+&
                  (loss_gain(ipts,ivm)+excess_veg_add+excess_veg_rem)) .GT. min_vegfrac)THEN
                ! Also, our total loss_gain needs to be above min_stomate
                IF(ABS(loss_gain(ipts,ivm)+excess_veg_add+excess_veg_rem) .GT. min_stomate)THEN
                   
                   IF(ld_lcc .AND. test_grid == ipts)THEN
                      WRITE(numout,*) 'Dumping excess vegetation'
                      WRITE(numout,*) 'ipts,ivm,ivma: ',ipts,ivm,ivma
                      WRITE(numout,*) 'Correction: ',&
                           excess_veg_add+excess_veg_rem
                      WRITE(numout,*) 'veget_max: ',veget_max(ipts,ivm)
                      WRITE(numout,*) 'veget_max corrected: ',veget_max(ipts,ivm)+&
                           (loss_gain(ipts,ivm)+excess_veg_add+excess_veg_rem)
                      WRITE(numout,*) 'loss_gain: ',loss_gain(ipts,ivm)
                      WRITE(numout,*) 'loss_gain corrected: ',loss_gain(ipts,ivm)+&
                           excess_veg_add+excess_veg_rem
                   ENDIF
                   ! Let's do it.
                   loss_gain(ipts,ivm)=loss_gain(ipts,ivm)+&
                        excess_veg_add+excess_veg_rem
                   excess_veg_add=zero
                   excess_veg_rem=zero
                   EXIT excess
                ENDIF
             ENDIF
          ENDIF
       ENDDO excess

       ! If we still have excess vegetation, our method did not work.
       IF((excess_veg_add .GE. min_stomate) .OR. &
            (ABS(excess_veg_rem) .GE. min_stomate))THEN

          ! It is possible that the excess amount we have to add and that which we have to remove
          ! cancel each other out.  In which case, we don't actually have a problem.
          IF(ABS(excess_veg_add+excess_veg_rem) .GE. min_stomate)THEN
             WRITE(numout,*) 'We were not able to redistribute all the extra vegetation in LCC.'
             WRITE(numout,*) 'excess_veg_add: ',excess_veg_add
             WRITE(numout,*) 'excess_veg_rem: ',excess_veg_rem
             WRITE(numout,*) 'ipts: ',ipts
             CALL print_lcc_error_message(npts,ipts,veget_max_new,veget_max,veget_max_new_group,&
                  veget_max_group,delta_veget,loss_gain,4)
             IF(lignore_lcc_stops)THEN
                CALL ipslerr_p (2,'land_cover_main', 'Not stopping here.',&
                     'Only ignore me if you know what you are doing!','Place 2')
                excess_veg_add=zero
                excess_veg_rem=zero
             ELSE
                CALL ipslerr_p (3,'land_cover_main', 'Unable to redistribute vegetation (see output file)','','')
             ENDIF

          ELSE
             IF(ld_lcc .AND. ipts == test_grid)THEN
                WRITE(numout,*) 'All extra vegetation successfully redistributed in LCC.'
                WRITE(numout,*) 'excess_veg_add: ',excess_veg_add
                WRITE(numout,*) 'excess_veg_rem: ',excess_veg_rem
             ENDIF
             excess_veg_add=zero
             excess_veg_rem=zero
          ENDIF
       ENDIF

       ! Reconstruct the delta_veget from the adjusted loss_gain array.
       delta_veget(ipts,:)=zero
       DO ivma=1,nvmap

          iyoung=start_index(ivma)
          nagec_local=nagec_pft(ivma)

          ! Add all these age classes to this map PFT
          DO ivm=iyoung,iyoung+nagec_local-1
             delta_veget(ipts,ivma)=delta_veget(ipts,ivma)+loss_gain(ipts,ivm)
          ENDDO

       ENDDO ! done looping over map PFTs



       ! Now one final sanity check.  We should never try to remove more
       ! vegetation than we actually have.  
       DO ivm=1,nvm
          ! Notice here that loss_gain is negative for pixels
          ! which remove veget_max.
          difference=veget_max(ipts,ivm)+loss_gain(ipts,ivm)
          IF(difference .LT. -min_stomate)THEN
             ! This is a problem.
             WRITE(numout,*) 'We are trying to remove too much vegetation in LCC.'
             WRITE(numout,*) 'ipts, ivm: ',ipts, ivm
             WRITE(numout,*) 'veget_max, loss_gain: ',&
                  veget_max(ipts,ivm),loss_gain(ipts,ivm)
             CALL ipslerr_p (3,'land_cover_main', 'Removing too much vegetation (see output file)','','')
             
          ELSEIF((difference .LE. min_vegfrac) .AND. &
               (difference .GT. min_stomate))THEN
             ! We should have accounted for this above.
             WRITE(numout,*) 'We are left with an awkward amount of vegetation vegetation in LCC.'
             WRITE(numout,*) 'This should have been taken care of above.'
             WRITE(numout,*) 'ipts, ivm: ',ipts, ivm
             CALL print_lcc_error_message(npts,test_grid,veget_max_new,veget_max,veget_max_new_group,&
                  veget_max_group,delta_veget,loss_gain,4)
             CALL ipslerr_p (3,'land_cover_main', 'Awkward amoutn of vegetation (see output file)','','')
          ELSEIF(difference .LT. zero)THEN
             ! Technically not a problem since it could be -1e-17, but it might provoke
             ! a mass balance error so we warn about it.  The serious case of
             ! a difference less than -min_stomate was handled above.
             WRITE(numout,*) 'WARNING: A small amount of leftover vegatation in LCC.'
             WRITE(numout,*) 'Should not cause a problem, but watch for a mass balance error.'
             WRITE(numout,*) 'ipts, ivm: ',ipts, ivm
             WRITE(numout,*) 'veget_max, loss_gain: ',&
                  veget_max(ipts,ivm),loss_gain(ipts,ivm)
             CALL ipslerr_p (2,'land_cover_main', 'Small amount of leftover vegetation (see output file)','','')
          ENDIF

       ENDDO ! loop over PFTs

    ENDDO ! loop over #pixels - domain size

  END SUBROUTINE adjust_delta_veget

END MODULE sapiens_lcchange
